/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file clk.c
 **
 ** A detailed description is available at 
 ** @link ClkGroup Clock Module description @endlink
 **
 ** History:
 **   - 2014-02-24  1.0  Edison Zhang  First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "clk.h"

#if (defined(PDL_PERIPHERAL_CLK_ACTIVE))

/**
 ******************************************************************************
 ** \addtogroup ClkGroup
 ******************************************************************************/
//@{

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_CLK == PDL_ON)
stc_clk_intern_data_t stcClkInternData; ///< Store the internal data of clock
#endif

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/

/**
 ******************************************************************************
 ** \brief Clock Stabilization Interrupt Handler
 ******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_CLK == PDL_ON)
void Clk_IrqHandler(void)
{
  uint8_t u8IntStrReadOut;
  
  u8IntStrReadOut = FM0P_CRG->INT_STR;
  
  // PLL stabilization ready?
  if (0u != (u8IntStrReadOut & FM0P_INT_STR_PCSI_BITPOS))
  {
    FM0P_CRG->INT_CLR |= FM0P_INT_CLR_PCSC_BITPOS;  // Clear Irq
    
    // PLL ready callback if defined
    if (stcClkInternData.pfnPllStabCb != NULL)
    {
      stcClkInternData.pfnPllStabCb();
    }
  }

  // Sub Clock stabilization ready?  
  if (0u != (u8IntStrReadOut & FM0P_INT_STR_SCSI_BITPOS))
  {
    FM0P_CRG->INT_CLR |= FM0P_INT_CLR_SCSC_BITPOS;  // Clear Irq
    
    // Sub Clock ready callback if defined
    if (stcClkInternData.pfnScoStabCb != NULL)
    {
      stcClkInternData.pfnScoStabCb();
    }
  }

  // Main Clock stabilization ready?  
  if (0u != (u8IntStrReadOut & FM0P_INT_STR_MCSI_BITPOS))
  {
    FM0P_CRG->INT_CLR |= FM0P_INT_CLR_MCSC_BITPOS;  // Clear Irq
    
    // Main Clock ready callback if defined
    if (stcClkInternData.pfnMcoStabCb != NULL)
    {
      stcClkInternData.pfnMcoStabCb();
    }
  }
}
#endif

/**
 ******************************************************************************
 ** \brief Initialize system clock according to user configuration
 **
 ** This function configues clock division, stability time, PLL setting.
 **
 ** \param [in]  pstcClk         Pointer to clock configuration structure
 **
 ** \retval Ok    Clock initialized normally
 ** \retval ErrorInvalidParameter    The parameter is set to error range    
 **
 ** \note  Set the definition CLOCK_SETUP to "CLOCK_SETTING_NONE" when using 
 **        this function to initialize system clock.
 ******************************************************************************/
en_result_t Clk_Init(stc_clk_config_t* pstcClk) 
{
    if(pstcClk == NULL)
    {
        return ErrorInvalidParameter;
    }
    
    /* Set base clock divider */
    switch(pstcClk->enBaseClkDiv)
    {
        case BaseClkDiv1:
            FM0P_CRG->BSC_PSR_f.BSR = 0u;
            break;
        case BaseClkDiv2:
            FM0P_CRG->BSC_PSR_f.BSR = 1u;
            break;    
        case BaseClkDiv3:
            FM0P_CRG->BSC_PSR_f.BSR = 2u;
            break;
        case BaseClkDiv4:
            FM0P_CRG->BSC_PSR_f.BSR = 3u;
            break;
        case BaseClkDiv6:
            FM0P_CRG->BSC_PSR_f.BSR = 4u;
            break;
        case BaseClkDiv8:
            FM0P_CRG->BSC_PSR_f.BSR = 5u;
            break;
        case BaseClkDiv16:
            FM0P_CRG->BSC_PSR_f.BSR = 6u;
            break;    
        default:
            return ErrorInvalidParameter;
            
    }
    
    /* Set APB0 bus clock dividor */
    switch(pstcClk->enAPB0Div)
    {
        case Apb0Div1:
            FM0P_CRG->APBC0_PSR_f.APBC0 = 0u;
            break;
        case Apb0Div2:
            FM0P_CRG->APBC0_PSR_f.APBC0 = 1u;
            break;
        case Apb0Div4:
            FM0P_CRG->APBC0_PSR_f.APBC0 = 2u;
            break;
        case Apb0Div8:
            FM0P_CRG->APBC0_PSR_f.APBC0 = 3u;
            break;    
        default:
            return ErrorInvalidParameter;
    }
    
    /* Set APB1 bus clock divider */
    switch(pstcClk->enAPB1Div)
    {
        case Apb1Div1:
            FM0P_CRG->APBC1_PSR_f.APBC1 = 0u;
            break;
        case Apb1Div2:
            FM0P_CRG->APBC1_PSR_f.APBC1 = 1u;
            break;
        case Apb1Div4:
            FM0P_CRG->APBC1_PSR_f.APBC1 = 2u;
            break;
        case Apb1Div8:
            FM0P_CRG->APBC1_PSR_f.APBC1 = 3u;
            break;    
        default:
            return ErrorInvalidParameter;
    }
    
    if(pstcClk->bAPB1Disable == TRUE)
    {
        FM0P_CRG->APBC1_PSR_f.APBC1EN = 0;
    }
    
    /* Configure stability wait time */
    FM0P_CRG->CSW_TMR_f.MOWT = pstcClk->enMCOWaitTime;
    FM0P_CRG->CSW_TMR_f.SOWT = pstcClk->enSCOWaitTime;
    FM0P_CRG->PSW_TMR_f.POWT = pstcClk->enPLLOWaitTime;
    
#if (PDL_INTERRUPT_ENABLE_CLK == PDL_ON)    
    /* Configure interrupt */
    if(pstcClk->bMcoIrq == TRUE)
    {
        if(pstcClk->pfnMcoStabCb == NULL)
        {
            return ErrorInvalidParameter;
        }
        
        FM0P_CRG->INT_ENR_f.MCSE = 1;
        stcClkInternData.pfnMcoStabCb = pstcClk->pfnMcoStabCb;
    }
    
    if(pstcClk->bScoIrq == TRUE)
    {
        if(pstcClk->pfnScoStabCb == NULL)
        {
            return ErrorInvalidParameter;
        }
        
        FM0P_CRG->INT_ENR_f.SCSE = 1;
        stcClkInternData.pfnScoStabCb = pstcClk->pfnScoStabCb;
    }
    
    if(pstcClk->bPllIrq == TRUE)
    {
        if(pstcClk->pfnPllStabCb == NULL)
        {
            return ErrorInvalidParameter;
        }
        
        FM0P_CRG->INT_ENR_f.PCSE = 1;
        stcClkInternData.pfnPllStabCb = pstcClk->pfnPllStabCb;
    }
#endif    
    
    /* Set PLL K, M, N */
    FM0P_CRG->PLL_CTL1_f.PLLK = pstcClk->u8PllK - 1;
    FM0P_CRG->PLL_CTL1_f.PLLM = pstcClk->u8PllM - 1;
    FM0P_CRG->PLL_CTL2_f.PLLN = pstcClk->u8PllN - 1;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Enable high speed CR
 **
 ** This function easily enables the high speed CR. No configuration is needed.
 **
 ** \param bBlock    Wait until CR stability or not
 ** \arg   FALSE     Return immediately after enable high speed CR
 ** \arg   TRUE      Wait until CR stability after enable high speed CR
 **
 ** \retval Ok                high speed CR enabled
 ******************************************************************************/
en_result_t Clk_EnableHscr(boolean_t bBlock)
{
  FM0P_CRG->SCM_CTL_f.HCRE = 1u;
  
  if(bBlock == TRUE)
  {
    while(FM0P_CRG->SCM_STR_f.HCRDY != 1);
  }

  return Ok;
} // Clk_EnableMainClock

/**
 ******************************************************************************
 ** \brief Disable Main Clock
 **
 ** This function easily disables the Main Clock. No configuration is needed.
 **
 ** \retval Ok                Main Clock disabled
 ******************************************************************************/
en_result_t Clk_DisableHscr(void)
{
  FM0P_CRG->SCM_CTL_f.MOSCE = 0u;

  return Ok;
} // Clk_DisableMainClock


/**
 ******************************************************************************
 ** \brief Enable Main Clock and wait until it is stable
 **
 ** This function easily enables the Main Clock. No configuration is needed.
 ** 
 ** \param bBlock    Wait until Main Clock stability or not
 ** \arg   FALSE     Return immediately after enable Main Clock
 ** \arg   TRUE      Wait until Main Clock stability after enable Main Clock
 **
 ** \retval Ok                Main Clock enabled
 ******************************************************************************/
en_result_t Clk_EnableMainClock(boolean_t bBlock)
{
  FM0P_CRG->SCM_CTL_f.MOSCE = 1u;
  
  if(bBlock == TRUE)
  {
    while(FM0P_CRG->SCM_STR_f.MORDY != 1);
  }

  return Ok;
} // Clk_EnableMainClock

/**
 ******************************************************************************
 ** \brief Disable Main Clock
 **
 ** This function easily disables the Main Clock. No configuration is needed.
 **
 ** \retval Ok                Main Clock disabled
 ******************************************************************************/
en_result_t Clk_DisableMainClock(void)
{
  FM0P_CRG->SCM_CTL_f.MOSCE = 0u;

  return Ok;
} // Clk_DisableMainClock


/**
 ******************************************************************************
 ** \brief Enable Sub Clock
 **
 ** This function easily enables the Sub Clock. No configuration is needed.
 **
 ** \param bBlock    Wait until Sub Clock stability or not
 ** \arg   FALSE     Return immediately after enable Sub Clock
 ** \arg   TRUE      Wait until Sub Clock stability after enable Sub Clock
 **
 ** \retval Ok                Sub Clock enabled
 ******************************************************************************/
en_result_t Clk_EnableSubClock(boolean_t bBlock)
{
  FM0P_CRG->SCM_CTL_f.SOSCE = 1u;
  
  if(bBlock == TRUE)
  {
    while(FM0P_CRG->SCM_STR_f.SORDY != 1);
  }

  return Ok;
} // Clk_EnableSubClock

/**
 ******************************************************************************
 ** \brief Disable Sub Clock
 **
 ** This function easily disables the Sub Clock. No configuration is needed.
 **
 ** \retval Ok                Sub Clock disabled
 ******************************************************************************/
en_result_t Clk_DisableSubClock(void)
{
  FM0P_CRG->SCM_CTL_f.SOSCE = 0u;

  return Ok;
} // Clk_DisableSubClock


/**
 ******************************************************************************
 ** \brief Enable PLL Clock
 **
 ** This function easily enables the PLL Clock. No configuration is needed.
 **
 ** \param bBlock    Wait until PLL Clock stability or not
 ** \arg   FALSE     Return immediately after enable PLL Clock
 ** \arg   TRUE      Wait until PLL Clock stability after enable PLL Clock
 **
 ** \retval Ok                PLL Clock enabled
 ******************************************************************************/
en_result_t Clk_EnablePllClock(boolean_t bBlock)
{
  FM0P_CRG->SCM_CTL_f.PLLE = 1u;
  
  if(bBlock == TRUE)
  {
    while(FM0P_CRG->SCM_STR_f.PLRDY != 1);
  }

  return Ok;
} // Clk_EnableSubClock

/**
 ******************************************************************************
 ** \brief Disable PLL Clock
 **
 ** This function easily disables the PLL Clock. No configuration is needed.
 **
 ** \retval Ok                PLL Clock disabled
 ******************************************************************************/
en_result_t Clk_DisablePllClock(void)
{
  FM0P_CRG->SCM_CTL_f.PLLE = 0u;

  return Ok;
} // Clk_DisableSubClock

/**
 ******************************************************************************
 ** \brief Set Clock Source
 **
 ** This function sets the clock source and performs transition, if wanted.
 **
 ** \param [in]  enSource        System source clock
 ** \arg         ClkMain         Set Main Clock as system source clock
 ** \arg         ClkSub          Set Sub Clock as system source clock
 ** \arg         ClkHsCr         Set High-speed CR as system source clock
 ** \arg         ClkLsCr         Set Low-speed CR as system source clock
 ** \arg         ClkPll          Set Main PLL clock as system source clock
 ** \arg         ClkHsCrPll      Set High-speed CR PLL clock as system source clock
 ** 
 ** \retval Ok                     Clock source set
 ** \retval ErrorInvalidParameter  pstcConfig == NULL or Illegal mode
 ** \retval ErrorInvalidMode       Clock setting not possible
 ******************************************************************************/
en_result_t Clk_SetSource(en_clk_source_t enSource)
{  
  uint8_t u8Rcs, u8Rcm;
  switch(enSource)
  {
    case ClkMain:
      if ((FM0P_CRG->SCM_CTL_f.MOSCE != TRUE) ||  // Main Oscillator ready?
          (FM0P_CRG->SCM_STR_f.MORDY != TRUE))
      {
        return ErrorInvalidMode ;
      }
      FM0P_CRG->SCM_CTL_f.RCS = 0x1u;
      break;
    case ClkSub:
      if ((FM0P_CRG->SCM_CTL_f.SOSCE != TRUE) ||  // Sub Oscillator ready?
          (FM0P_CRG->SCM_STR_f.SORDY != TRUE))
      {
        return ErrorInvalidMode ;
      }
      FM0P_CRG->SCM_CTL_f.RCS = 0x5u;
      break;
    case ClkHsCr:                                // Always possible
      if(FM0P_CRG->SCM_STR_f.HCRDY != TRUE)
      {
        return ErrorInvalidMode ;
      }
      FM0P_CRG->SCM_CTL_f.RCS = 0x0u;
      break;
    case ClkLsCr:                                // Always possible
      FM0P_CRG->SCM_CTL_f.RCS = 0x4u;
      break;
    case ClkHsCrPll:  
      if(FM0P_CRG->SCM_STR_f.HCRDY != TRUE)
      {
        return ErrorInvalidMode ;
      }
      FM0P_CRG->PSW_TMR_f.PINC = 1;
      FM0P_CRG->SCM_CTL_f.RCS = 0x2u;
      break;
    case ClkPll:                           
      if ((FM0P_CRG->SCM_STR_f.MORDY != TRUE) ||  // PLL ready?
          (FM0P_CRG->SCM_STR_f.PLRDY != TRUE))
      {
        return ErrorInvalidMode ;
      }
      
      FM0P_CRG->PSW_TMR_f.PINC = 0;
      FM0P_CRG->SCM_CTL_f.RCS = 0x2u;
      break;
    default:
      return ErrorInvalidParameter ;
  }

  /* Wait until switch stable */
  while(1)
  {
    u8Rcs = FM0P_CRG->SCM_CTL_f.RCS;
    u8Rcm = FM0P_CRG->SCM_STR_f.RCM;
    if(u8Rcs == u8Rcm)
    {
        break;
    }
  }

  return Ok;
} // Clk_SetSource


/**
 ******************************************************************************
 ** \brief Enables the clock gate of a peripheral
 **
 ** This function sets the corresponding bit in the CKENn register to enable
 ** the clock of a peripheral.
 **
 ** \param  enPeripheral           Enumerator of a peripheral, see
 **                                #en_clk_gate_peripheral_t for details
 **
 ** \retval Ok                     Peripheral clock enabled
 ** \retval ErrorInvalidParameter  Peripheral enumerator does not exist
 ******************************************************************************/
en_result_t Clk_PeripheralClockEnable(en_clk_gate_peripheral_t enPeripheral)
{
  switch (enPeripheral)
  {
    case ClkGateGpio:
      FM0P_PCG->CKEN0_f.GIOCK = 1u;
      break;
    case ClkGateDma:
      FM0P_PCG->CKEN0_f.DMACK = 1u;
      break;
    case ClkGateAdc0:
      FM0P_PCG->CKEN0_f.ADCCK0 = 1u;
      break;
    case ClkGateAdc1:
      FM0P_PCG->CKEN0_f.ADCCK1 = 1u;
      break;
    case ClkGateAdc2:
      FM0P_PCG->CKEN0_f.ADCCK2 = 1u;
      break;
    case ClkGateAdc3:
      FM0P_PCG->CKEN0_f.ADCCK3 = 1u;
      break;
    case ClkGateMfs0:
      FM0P_PCG->CKEN0_f.MFSCK0 = 1u;
      break;
    case ClkGateMfs1:
      FM0P_PCG->CKEN0_f.MFSCK1 = 1u;
      break;
    case ClkGateMfs2:
      FM0P_PCG->CKEN0_f.MFSCK2 = 1u;
      break;
    case ClkGateMfs3:
      FM0P_PCG->CKEN0_f.MFSCK3 = 1u;
      break;
    case ClkGateMfs4:
      FM0P_PCG->CKEN0_f.MFSCK4 = 1u;
      break;
    case ClkGateMfs5:
      FM0P_PCG->CKEN0_f.MFSCK5 = 1u;
      break;
    case ClkGateMfs6:
      FM0P_PCG->CKEN0_f.MFSCK6 = 1u;
      break;
    case ClkGateMfs7:
      FM0P_PCG->CKEN0_f.MFSCK7 = 1u;
      break;
    case ClkGateMfs8:
      FM0P_PCG->CKEN0_f.MFSCK8 = 1u;
      break;
    case ClkGateMfs9:
      FM0P_PCG->CKEN0_f.MFSCK9 = 1u;
      break;
    case ClkGateMfs10:
      FM0P_PCG->CKEN0_f.MFSCK10 = 1u;
      break;
    case ClkGateMfs11:
      FM0P_PCG->CKEN0_f.MFSCK11 = 1u;
      break;
    case ClkGateMfs12:
      FM0P_PCG->CKEN0_f.MFSCK12 = 1u;
      break;
    case ClkGateMfs13:
      FM0P_PCG->CKEN0_f.MFSCK13 = 1u;
      break;
    case ClkGateMfs14:
      FM0P_PCG->CKEN0_f.MFSCK14 = 1u;
      break;
    case ClkGateMfs15:
      FM0P_PCG->CKEN0_f.MFSCK15 = 1u;
      break;
    case ClkGateQprc0:
      FM0P_PCG->CKEN1_f.QDUCK0 = 1u;
      break;
    case ClkGateQprc1:
      FM0P_PCG->CKEN1_f.QDUCK1 = 1u;
      break;
    case ClkGateQprc2:
      FM0P_PCG->CKEN1_f.QDUCK2 = 1u;
      break;
    case ClkGateQprc3:
      FM0P_PCG->CKEN1_f.QDUCK3 = 1u;
      break;
    case ClkGateMft0:
      FM0P_PCG->CKEN1_f.MFTCK0 = 1u;
      break;
    case ClkGateMft1:
      FM0P_PCG->CKEN1_f.MFTCK1 = 1u;
      break;
    case ClkGateMft2:
      FM0P_PCG->CKEN1_f.MFTCK2 = 1u;
      break;
    case ClkGateMft3:
      FM0P_PCG->CKEN1_f.MFTCK3 = 1u;
      break;
    case ClkGateBt0:
      FM0P_PCG->CKEN1_f.BTMCK0 = 1u;
      break;
    case ClkGateBt4:
      FM0P_PCG->CKEN1_f.BTMCK1 = 1u;
      break;
    case ClkGateBt8:
      FM0P_PCG->CKEN1_f.BTMCK2 = 1u;
      break;
    case ClkGateBt12:
      FM0P_PCG->CKEN1_f.BTMCK3 = 1u;
      break;
    case ClkGateCan0:
      FM0P_PCG->CKEN2_f.CANCK0 = 1u;
      break;
    case ClkGateCan1:
      FM0P_PCG->CKEN2_f.CANCK1 = 1u;
      break;
    default:
      return ErrorInvalidParameter;
  }
  
  return Ok;
} // Clk_PeripheralClockEnable

/**
 ******************************************************************************
 ** \brief Read the clock gate state of a peripheral
 **
 ** This function reads out the corresponding bit in the CKENn register.
 **
 ** \param  enPeripheral           Enumerator of a peripheral, see
 **                                #en_clk_gate_peripheral_t for details
 **
 ** \retval TRUE                   Peripheral clock enabled
 ** \retval FALSE                  Peripheral clock not enabled, peripheral
 **                                not existing
 ******************************************************************************/
boolean_t Clk_PeripheralGetClockState(en_clk_gate_peripheral_t enPeripheral)
{
  switch (enPeripheral)
  {
    case ClkGateGpio:
      return ((1u == FM0P_PCG->CKEN0_f.GIOCK) ? TRUE : FALSE);
    case ClkGateDma:
      return ((1u == FM0P_PCG->CKEN0_f.DMACK) ? TRUE : FALSE);
    case ClkGateAdc0:
      return ((1u == FM0P_PCG->CKEN0_f.ADCCK0) ? TRUE : FALSE);
    case ClkGateAdc1:
      return ((1u == FM0P_PCG->CKEN0_f.ADCCK1) ? TRUE : FALSE);
    case ClkGateAdc2:
      return ((1u == FM0P_PCG->CKEN0_f.ADCCK2) ? TRUE : FALSE);
    case ClkGateAdc3:
      return ((1u == FM0P_PCG->CKEN0_f.ADCCK3) ? TRUE : FALSE);
    case ClkGateMfs0:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK0) ? TRUE : FALSE);
    case ClkGateMfs1:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK1) ? TRUE : FALSE);
    case ClkGateMfs2:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK2) ? TRUE : FALSE);
    case ClkGateMfs3:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK3) ? TRUE : FALSE);
    case ClkGateMfs4:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK4) ? TRUE : FALSE);
    case ClkGateMfs5:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK5) ? TRUE : FALSE);
    case ClkGateMfs6:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK6) ? TRUE : FALSE);
    case ClkGateMfs7:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK7) ? TRUE : FALSE);
    case ClkGateMfs8:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK8) ? TRUE : FALSE);
    case ClkGateMfs9:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK9) ? TRUE : FALSE);
    case ClkGateMfs10:
      return ((1u ==  FM0P_PCG->CKEN0_f.MFSCK10) ? TRUE : FALSE);
    case ClkGateMfs11:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK11) ? TRUE : FALSE);
    case ClkGateMfs12:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK12) ? TRUE : FALSE);
    case ClkGateMfs13:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK13) ? TRUE : FALSE);
    case ClkGateMfs14:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK14) ? TRUE : FALSE);
    case ClkGateMfs15:
      return ((1u == FM0P_PCG->CKEN0_f.MFSCK15) ? TRUE : FALSE);
    case ClkGateQprc0:
      return ((1u == FM0P_PCG->CKEN1_f.QDUCK0) ? TRUE : FALSE);
    case ClkGateQprc1:
      return ((1u == FM0P_PCG->CKEN1_f.QDUCK1) ? TRUE : FALSE);
    case ClkGateQprc2:
      return ((1u == FM0P_PCG->CKEN1_f.QDUCK2) ? TRUE : FALSE);
    case ClkGateQprc3:
      return ((1u == FM0P_PCG->CKEN1_f.QDUCK3) ? TRUE : FALSE);
    case ClkGateMft0:
      return ((1u == FM0P_PCG->CKEN1_f.MFTCK0) ? TRUE : FALSE);
    case ClkGateMft1:
      return ((1u == FM0P_PCG->CKEN1_f.MFTCK1) ? TRUE : FALSE);
    case ClkGateMft2:
      return ((1u == FM0P_PCG->CKEN1_f.MFTCK2) ? TRUE : FALSE);
    case ClkGateMft3:
      return ((1u == FM0P_PCG->CKEN1_f.MFTCK3) ? TRUE : FALSE);
    case ClkGateBt0:
      return ((1u == FM0P_PCG->CKEN1_f.BTMCK0) ? TRUE : FALSE);
    case ClkGateBt4:
      return ((1u == FM0P_PCG->CKEN1_f.BTMCK1) ? TRUE : FALSE);
    case ClkGateBt8:
      return ((1u == FM0P_PCG->CKEN1_f.BTMCK2) ? TRUE : FALSE);
    case ClkGateBt12:
      return ((1u == FM0P_PCG->CKEN1_f.BTMCK3) ? TRUE : FALSE);
    case ClkGateCan0:
      return ((1u == FM0P_PCG->CKEN2_f.CANCK0) ? TRUE : FALSE);
    case ClkGateCan1:
      return ((1u == FM0P_PCG->CKEN2_f.CANCK1) ? TRUE : FALSE);
    default:
      break;
  }
  
  return FALSE; // Peripheral not found -> always FALSE
} // Clk_PeripheralGetClockState

/**
 ******************************************************************************
 ** \brief Disables the clock gate of a peripheral
 **
 ** This function clears the corresponding bit in the CKENn register to disable
 ** the clock of a peripheral.
 **
 ** \param  enPeripheral           Enumerator of a peripheral, see
 **                                #en_clk_gate_peripheral_t for details
 **
 ** \retval Ok                     Peripheral clock disabled
 ** \retval ErrorInvalidParameter  Peripheral enumerator does not exist
 ******************************************************************************/
en_result_t Clk_PeripheralClockDisable(en_clk_gate_peripheral_t enPeripheral)
{
  switch (enPeripheral)
  {
    case ClkGateGpio:
      FM0P_PCG->CKEN0_f.GIOCK = 0u;
      break;
    case ClkGateDma:
      FM0P_PCG->CKEN0_f.DMACK = 0u;
      break;
    case ClkGateAdc0:
      FM0P_PCG->CKEN0_f.ADCCK0 = 0u;
      break;
    case ClkGateAdc1:
      FM0P_PCG->CKEN0_f.ADCCK1 = 0u;
      break;
    case ClkGateAdc2:
      FM0P_PCG->CKEN0_f.ADCCK2 = 0u;
      break;
    case ClkGateAdc3:
      FM0P_PCG->CKEN0_f.ADCCK3 = 0u;
      break;
    case ClkGateMfs0:
      FM0P_PCG->CKEN0_f.MFSCK0 = 0u;
      break;
    case ClkGateMfs1:
      FM0P_PCG->CKEN0_f.MFSCK1 = 0u;
      break;
    case ClkGateMfs2:
      FM0P_PCG->CKEN0_f.MFSCK2 = 0u;
      break;
    case ClkGateMfs3:
      FM0P_PCG->CKEN0_f.MFSCK3 = 0u;
      break;
    case ClkGateMfs4:
      FM0P_PCG->CKEN0_f.MFSCK4 = 0u;
      break;
    case ClkGateMfs5:
      FM0P_PCG->CKEN0_f.MFSCK5 = 0u;
      break;
    case ClkGateMfs6:
      FM0P_PCG->CKEN0_f.MFSCK6 = 0u;
      break;
    case ClkGateMfs7:
      FM0P_PCG->CKEN0_f.MFSCK7 = 0u;
      break;
    case ClkGateMfs8:
      FM0P_PCG->CKEN0_f.MFSCK8 = 0u;
      break;
    case ClkGateMfs9:
      FM0P_PCG->CKEN0_f.MFSCK9 = 0u;
      break;
    case ClkGateMfs10:
      FM0P_PCG->CKEN0_f.MFSCK10 = 0u;
      break;
    case ClkGateMfs11:
      FM0P_PCG->CKEN0_f.MFSCK11 = 0u;
      break;
    case ClkGateMfs12:
      FM0P_PCG->CKEN0_f.MFSCK12 = 0u;
      break;
    case ClkGateMfs13:
      FM0P_PCG->CKEN0_f.MFSCK13 = 0u;
      break;
    case ClkGateMfs14:
      FM0P_PCG->CKEN0_f.MFSCK14 = 0u;
      break;
    case ClkGateMfs15:
      FM0P_PCG->CKEN0_f.MFSCK15 = 0u;
      break;
    case ClkGateQprc0:
      FM0P_PCG->CKEN1_f.QDUCK0 = 0u;
      break;
    case ClkGateQprc1:
      FM0P_PCG->CKEN1_f.QDUCK1 = 0u;
      break;
    case ClkGateQprc2:
      FM0P_PCG->CKEN1_f.QDUCK2 = 0u;
      break;
    case ClkGateQprc3:
      FM0P_PCG->CKEN1_f.QDUCK3 = 0u;
      break;
    case ClkGateMft0:
      FM0P_PCG->CKEN1_f.MFTCK0 = 0u;
      break;
    case ClkGateMft1:
      FM0P_PCG->CKEN1_f.MFTCK1 = 0u;
      break;
    case ClkGateMft2:
      FM0P_PCG->CKEN1_f.MFTCK2 = 0u;
      break;
    case ClkGateMft3:
      FM0P_PCG->CKEN1_f.MFTCK3 = 0u;
      break;
    case ClkGateBt0:
      FM0P_PCG->CKEN1_f.BTMCK0 = 0u;
      break;
    case ClkGateBt4:
      FM0P_PCG->CKEN1_f.BTMCK1 = 0u;
      break;
    case ClkGateBt8:
      FM0P_PCG->CKEN1_f.BTMCK2 = 0u;
      break;
    case ClkGateBt12:
      FM0P_PCG->CKEN1_f.BTMCK3 = 0u;
      break;
    case ClkGateCan0:
      FM0P_PCG->CKEN2_f.CANCK0 = 0u;
      break;
    case ClkGateCan1:
      FM0P_PCG->CKEN2_f.CANCK1 = 0u;
      break;
    default:
      return ErrorInvalidParameter;
  }
  
  return Ok;
} // Clk_PeripheralClockDisable

/**
 ******************************************************************************
 ** \brief Set reset bit a peripheral
 **
 ** This function sets the corresponding bit in the MRSTn register to set
 ** a peripheral in reset state.
 **
 ** \param  enPeripheral           Enumerator of a peripheral, see
 **                                #en_clk_reset_peripheral_t for details
 **
 ** \retval Ok                     Peripheral clock enabled
 ** \retval ErrorInvalidParameter  Peripheral enumerator does not exist
 ******************************************************************************/
en_result_t Clk_PeripheralSetReset(en_clk_reset_peripheral_t enPeripheral)
{
  switch (enPeripheral)
  {
    case ClkResetDma:
      FM0P_PCG->MRST0_f.DMARST = 1u;
      break;
    case ClkResetAdc0:
      FM0P_PCG->MRST0_f.ADCRST0 = 1u;
      break;
    case ClkResetAdc1:
      FM0P_PCG->MRST0_f.ADCRST1 = 1u;
      break;
    case ClkResetAdc2:
      FM0P_PCG->MRST0_f.ADCRST2 = 1u;
      break;
    case ClkResetAdc3:
      FM0P_PCG->MRST0_f.ADCRST3 = 1u;
      break;
    case ClkResetMfs0:
      FM0P_PCG->MRST0_f.MFSRST0 = 1u;
      break;
    case ClkResetMfs1:
      FM0P_PCG->MRST0_f.MFSRST1 = 1u;
      break;
    case ClkResetMfs2:
      FM0P_PCG->MRST0_f.MFSRST2 = 1u;
      break;
    case ClkResetMfs3:
      FM0P_PCG->MRST0_f.MFSRST3 = 1u;
      break;
    case ClkResetMfs4:
      FM0P_PCG->MRST0_f.MFSRST4 = 1u;
      break;
    case ClkResetMfs5:
      FM0P_PCG->MRST0_f.MFSRST5 = 1u;
      break;
    case ClkResetMfs6:
      FM0P_PCG->MRST0_f.MFSRST6 = 1u;
      break;
    case ClkResetMfs7:
      FM0P_PCG->MRST0_f.MFSRST7 = 1u;
      break;
    case ClkResetMfs8:
      FM0P_PCG->MRST0_f.MFSRST8 = 1u;
      break;
    case ClkResetMfs9:
      FM0P_PCG->MRST0_f.MFSRST9 = 1u;
      break;
    case ClkResetMfs10:
      FM0P_PCG->MRST0_f.MFSRST10 = 1u;
      break;
    case ClkResetMfs11:
      FM0P_PCG->MRST0_f.MFSRST11 = 1u;
      break;
    case ClkResetMfs12:
      FM0P_PCG->MRST0_f.MFSRST12 = 1u;
      break;
    case ClkResetMfs13:
      FM0P_PCG->MRST0_f.MFSRST13 = 1u;
      break;
    case ClkResetMfs14:
      FM0P_PCG->MRST0_f.MFSRST14 = 1u;
      break;
    case ClkResetMfs15:
      FM0P_PCG->MRST0_f.MFSRST15 = 1u;
      break;
    case ClkResetQprc0:
      FM0P_PCG->MRST1_f.QDURST0 = 1u;
      break;
    case ClkResetQprc1:
      FM0P_PCG->MRST1_f.QDURST1 = 1u;
      break;
    case ClkResetQprc2:
      FM0P_PCG->MRST1_f.QDURST2 = 1u;
      break;
    case ClkResetQprc3:
      FM0P_PCG->MRST1_f.QDURST3 = 1u;
      break;
    case ClkResetMft0:
      FM0P_PCG->MRST1_f.MFTRST0 = 1u;
      break;
    case ClkResetMft1:
      FM0P_PCG->MRST1_f.MFTRST1 = 1u;
      break;
    case ClkResetMft2:
      FM0P_PCG->MRST1_f.MFTRST2 = 1u;
      break;
    case ClkResetMft3:
      FM0P_PCG->MRST1_f.MFTRST3 = 1u;
      break;
    case ClkResetBt0:
      FM0P_PCG->MRST1_f.BTMRST0 = 1u;
      break;
    case ClkResetBt4:
      FM0P_PCG->MRST1_f.BTMRST1 = 1u;
      break;
    case ClkResetBt8:
      FM0P_PCG->MRST1_f.BTMRST2 = 1u;
      break;
    case ClkResetBt12:
      FM0P_PCG->MRST1_f.BTMRST3 = 1u;
      break;
    case ClkResetCan0:
      FM0P_PCG->MRST2_f.CANRST0 = 1u;
      break;
    case ClkResetCan1:
      FM0P_PCG->MRST2_f.CANRST1 = 1u;
      break;
    default:
      return ErrorInvalidParameter;
  }
  
  return Ok;
} // Clk_PeripheralSetReset

/**
 ******************************************************************************
 ** \brief Clear reset bit a peripheral
 **
 ** This function clears the corresponding bit in the MRSTn register to release
 ** a peripheral from reset state.
 **
 ** \param  enPeripheral           Enumerator of a peripheral, see
 **                                #en_clk_reset_peripheral_t for details
 **
 ** \retval Ok                     Peripheral clock enabled
 ** \retval ErrorInvalidParameter  Peripheral enumerator does not exist
 ******************************************************************************/
en_result_t Clk_PeripheralClearReset(en_clk_reset_peripheral_t enPeripheral)
{
  switch (enPeripheral)
  {
    case ClkResetDma:
      FM0P_PCG->MRST0_f.DMARST = 0u;
      break;
    case ClkResetAdc0:
      FM0P_PCG->MRST0_f.ADCRST0 = 0u;
      break;
    case ClkResetAdc1:
      FM0P_PCG->MRST0_f.ADCRST1 = 0u;
      break;
    case ClkResetAdc2:
      FM0P_PCG->MRST0_f.ADCRST2 = 0u;
      break;
    case ClkResetAdc3:
      FM0P_PCG->MRST0_f.ADCRST3 = 0u;
      break;
    case ClkResetMfs0:
      FM0P_PCG->MRST0_f.MFSRST0 = 0u;
      break;
    case ClkResetMfs1:
      FM0P_PCG->MRST0_f.MFSRST1 = 0u;
      break;
    case ClkResetMfs2:
      FM0P_PCG->MRST0_f.MFSRST2 = 0u;
      break;
    case ClkResetMfs3:
      FM0P_PCG->MRST0_f.MFSRST3 = 0u;
      break;
    case ClkResetMfs4:
      FM0P_PCG->MRST0_f.MFSRST4 = 0u;
      break;
    case ClkResetMfs5:
      FM0P_PCG->MRST0_f.MFSRST5 = 0u;
      break;
    case ClkResetMfs6:
      FM0P_PCG->MRST0_f.MFSRST6 = 0u;
      break;
    case ClkResetMfs7:
      FM0P_PCG->MRST0_f.MFSRST7 = 0u;
      break;
    case ClkResetMfs8:
      FM0P_PCG->MRST0_f.MFSRST8 = 0u;
      break;
    case ClkResetMfs9:
      FM0P_PCG->MRST0_f.MFSRST9 = 0u;
      break;
    case ClkResetMfs10:
      FM0P_PCG->MRST0_f.MFSRST10 = 0u;
      break;
    case ClkResetMfs11:
      FM0P_PCG->MRST0_f.MFSRST11 = 0u;
      break;
    case ClkResetMfs12:
      FM0P_PCG->MRST0_f.MFSRST12 = 0u;
      break;
    case ClkResetMfs13:
      FM0P_PCG->MRST0_f.MFSRST13 = 0u;
      break;
    case ClkResetMfs14:
      FM0P_PCG->MRST0_f.MFSRST14 = 0u;
      break;
    case ClkResetMfs15:
      FM0P_PCG->MRST0_f.MFSRST15 = 0u;
      break;
    case ClkResetQprc0:
      FM0P_PCG->MRST1_f.QDURST0 = 0u;
      break;
    case ClkResetQprc1:
      FM0P_PCG->MRST1_f.QDURST1 = 0u;
      break;
    case ClkResetQprc2:
      FM0P_PCG->MRST1_f.QDURST2 = 0u;
      break;
    case ClkResetQprc3:
      FM0P_PCG->MRST1_f.QDURST3 = 0u;
      break;
    case ClkResetMft0:
      FM0P_PCG->MRST1_f.MFTRST0 = 0u;
      break;
    case ClkResetMft1:
      FM0P_PCG->MRST1_f.MFTRST1 = 0u;
      break;
    case ClkResetMft2:
      FM0P_PCG->MRST1_f.MFTRST2 = 0u;
      break;
    case ClkResetMft3:
      FM0P_PCG->MRST1_f.MFTRST3 = 0u;
      break;
    case ClkResetBt0:
      FM0P_PCG->MRST1_f.BTMRST0 = 0u;
      break;
    case ClkResetBt4:
      FM0P_PCG->MRST1_f.BTMRST1 = 0u;
      break;
    case ClkResetBt8:
      FM0P_PCG->MRST1_f.BTMRST2 = 0u;
      break;
    case ClkResetBt12:
      FM0P_PCG->MRST1_f.BTMRST3 = 0u;
      break;
    case ClkResetCan0:
      FM0P_PCG->MRST2_f.CANRST0 = 0u;
      break;
    case ClkResetCan1:
      FM0P_PCG->MRST2_f.CANRST1 = 0u;
      break;
    default:
      return ErrorInvalidParameter;
  }
  
  return Ok;
} // Clk_PeripheralClearReset
       
//@} // ClkGroup

#endif // #if (defined(PDL_PERIPHERAL_ENABLE_CLK))

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
