/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file clk.h
 **
 ** Headerfile for CLK functions
 **  
 ** History:
 **   - 2014-02-24  1.0  Edison Zhang  First version.
 **
 ******************************************************************************/

#ifndef __CLK_H__
#define __CLK_H__

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mcu.h"
#include "pdl_user.h"

#if (defined(PDL_PERIPHERAL_CLK_ACTIVE))

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif
  
/**
 ******************************************************************************
 ** \defgroup ClkGroup Clock Functions (CLK)
 **  
 ** Provided CLK module functions:
 ** 
 ** - Clk_Init()
 ** - Clk_EnableHscr() 
 ** - Clk_DisableHscr()
 ** - Clk_EnableMainClock()
 ** - Clk_DisableMainClock()
 ** - Clk_EnableSubClock()
 ** - Clk_DisableSubClock()
 ** - Clk_EnablePllClock()
 ** - Clk_DisablePllClock()
 ** - Clk_SetSource()
 ** - Clk_PeripheralClockEnable()
 ** - Clk_PeripheralClockDisable()
 ** - Clk_PeripheralGetClockState()
 ** - Clk_PeripheralSetReset()
 ** - Clk_PeripheralClearReset()
 **  
 ** Clk_Init() initializes the system clock according to the configuration
 ** of the structure type #stc_clk_config_t. This function provides another
 ** method to configure system clock beside the CMSIS way.
 **
 ** Clk_EnableHscr() enables the high speed CR and Clk_DisableHscr() disables
 ** high speed CR.
 **
 ** Clk_EnableMainClock() enables main clock and Clk_DisableMainClock() disables
 ** main clock.
 **
 ** Clk_EnableSubClock() enables sub clock and Clk_DisableSubClock() disables
 ** sub clock.
 **
 ** Clk_EnablePllClock() enables PLL clock and Clk_DisablePllClock() disables
 ** PLL clock.  
 **  
 ** Clk_SetSource() set the source clock for system clock according to the 
 ** the enumeration type #en_clk_source_t. Before calling this function, the
 ** used clock should be enabled.
 **
 ** Clk_PeripheralClockEnable() enables the clock supply to the peripheral 
 ** selected and Clk_PeripheralClockDisable() disables the clock supply to 
 ** the peripheral selected. At default status, the clock is supplied.
 ** Clk_PeripheralGetClockState() reads the clock status of the peripheral 
 ** selected.
 **  
 ** Calling Clk_PeripheralSetReset() first and then Clk_PeripheralClearReset()
 ** can reset a peripheral.  
 **   
 ******************************************************************************/
//@{
  
/******************************************************************************
 * Global type definitions
 ******************************************************************************/
// PLL Input Clock Select (PINC) Bit Position
#define FM0P_PSW_TMR_PINC_BITPOS 0x10u
  
// Interrupt enable bit positions in INT_ENR
#define FM0P_INT_ENR_PCSE_BITPOS 0x04u
#define FM0P_INT_ENR_SCSE_BITPOS 0x02u
#define FM0P_INT_ENR_MCSE_BITPOS 0x01u
  
// Interrupt status bit positions in INT_STR
#define FM0P_INT_STR_PCSI_BITPOS 0x04u
#define FM0P_INT_STR_SCSI_BITPOS 0x02u
#define FM0P_INT_STR_MCSI_BITPOS 0x01u
  
// Interrupt clear bit positions in INT_CLR
#define FM0P_INT_CLR_PCSC_BITPOS 0x04u
#define FM0P_INT_CLR_SCSC_BITPOS 0x02u
#define FM0P_INT_CLR_MCSC_BITPOS 0x01u
  
/**
 ******************************************************************************
 ** \brief Clock Source
 ** 
 ** Differentiator for the different clock sources
 **
 ** \note The enumerated values do not correspond to the RCS/RCM bits of the
 **       clock control and status registers due to having upward compatibility,
 **       if this bit coding may change in future devices. The correct bit
 **       patterns are set by switch(en_clk_source)/case statements individually
 **       in the Clk_SetSource() function.
 ******************************************************************************/
typedef enum en_clk_source
{
  ClkMain = 0,  ///< Main Clock Oscillator
  ClkSub  = 1,  ///< Sub Clock Oscillator
  ClkHsCr = 2,  ///< High-Speed CR Clock Oscillator
  ClkLsCr = 3,  ///< Low-Speed CR Clock Oscillator
  ClkPll  = 4,  ///< PLL Clock
  ClkHsCrPll = 5   ///< High-Speed CR PLL Clock
} en_clk_source_t;

/**
 ******************************************************************************
 ** \brief Base Clock Prescaler Settings
 ** 
 ** Enumeration of the dividers of the Base Clock (HCLK)
 ******************************************************************************/
typedef enum en_clk_baseclkdiv
{
  BaseClkDiv1  = 0,  ///< HCLK Division 1/1
  BaseClkDiv2  = 1,  ///< HCLK Division 1/2
  BaseClkDiv3  = 2,  ///< HCLK Division 1/3
  BaseClkDiv4  = 3,  ///< HCLK Division 1/4
  BaseClkDiv6  = 4,  ///< HCLK Division 1/6
  BaseClkDiv8  = 5,  ///< HCLK Division 1/8    
  BaseClkDiv16 = 6,  ///< HCLK Division 1/16
  BaseClkErr   = 7   ///< HCLK prohibited setting
} en_clk_baseclkdiv_t;

/**
 ******************************************************************************
 ** \brief APB0 Prescaler Settings
 ** 
 ** Enumeration of the dividers of the APB0 (PCLK0)
 ******************************************************************************/
typedef enum en_clk_apb0div
{
  Apb0Div1  = 0,  ///< PCLK0 Division 1/1
  Apb0Div2  = 1,  ///< PCLK0 Division 1/2
  Apb0Div4  = 2,  ///< PCLK0 Division 1/4
  Apb0Div8  = 3   ///< PCLK0 Division 1/8
} en_clk_apb0div_t;

/**
 ******************************************************************************
 ** \brief APB1 Prescaler Settings
 ** 
 ** Enumeration of the dividers of the APB0 (PCLK1)
 ******************************************************************************/
typedef enum en_clk_apb1div
{
  Apb1Div1  = 0,  ///< PCLK1 Division 1/1
  Apb1Div2  = 1,  ///< PCLK1 Division 1/2
  Apb1Div4  = 2,  ///< PCLK1 Division 1/4
  Apb1Div8  = 3   ///< PCLK1 Division 1/8
} en_clk_apb1div_t;

/**
 ******************************************************************************
 ** \brief Sub Clock oscillation stabilization wait time
 ** 
 ** Enumeration for the Sub Clock oscillation stabilization wait time settings
 ******************************************************************************/
typedef enum en_clk_scowaittime
{
  ScoWaitExp10  =  0,  ///< 2^10 / F(CL) => ~10.3  ms
  ScoWaitExp11  =  1,  ///< 2^11 / F(CL) => ~20.5  ms
  ScoWaitExp12  =  2,  ///< 2^12 / F(CL) => ~41    ms
  ScoWaitExp13  =  3,  ///< 2^13 / F(CL) => ~82    ms
  ScoWaitExp14  =  4,  ///< 2^14 / F(CL) => ~164   ms
  ScoWaitExp15  =  5,  ///< 2^15 / F(CL) => ~327   ms
  ScoWaitExp16  =  6,  ///< 2^16 / F(CL) => ~655   ms
  ScoWaitExp17  =  7,  ///< 2^17 / F(CL) => ~1.31  s      
  ScoWaitExp18  =  8,  ///< 2^17 / F(CL) => ~2.62  s
  ScoWaitExp19  =  9,  ///< 2^17 / F(CL) => ~5.24  s 
  ScoWaitExp20  = 10,  ///< 2^17 / F(CL) => ~10.48 s
  ScoWaitExp21  = 11,  ///< 2^17 / F(CL) => ~20.96 s 
  ScoWaitErr    = 12   ///< Prohibited Setting
} en_clk_scowaittime_t;

/**
 ******************************************************************************
 ** \brief Main Clock oscillation stabilization wait time
 ** 
 ** Enumeration for the Main Clock oscillation stabilization wait time settings
 ******************************************************************************/
typedef enum en_clk_mcowaittime
{
  McoWaitExp11   = 0,   ///< 2^1  / F(CH) => ~500  ns (F(CH) = 4 MHz)
  McoWaitExp15   = 1,   ///< 2^5  / F(CH) => ~8    us (F(CH) = 4 MHz)
  McoWaitExp16   = 2,   ///< 2^6  / F(CH) => ~16   us (F(CH) = 4 MHz)
  McoWaitExp17   = 3,   ///< 2^7  / F(CH) => ~32   us (F(CH) = 4 MHz)
  McoWaitExp18   = 4,   ///< 2^8  / F(CH) => ~64   us (F(CH) = 4 MHz)
  McoWaitExp19   = 5,   ///< 2^9  / F(CH) => ~128  us (F(CH) = 4 MHz)
  McoWaitExp110  = 6,   ///< 2^10 / F(CH) => ~256  us (F(CH) = 4 MHz)
  McoWaitExp111  = 7,   ///< 2^11 / F(CH) => ~512  us (F(CH) = 4 MHz)
  McoWaitExp112  = 8,   ///< 2^12 / F(CH) => ~1.0  ms (F(CH) = 4 MHz)
  McoWaitExp113  = 9,   ///< 2^13 / F(CH) => ~2.0  ms (F(CH) = 4 MHz)
  McoWaitExp114  = 10,  ///< 2^14 / F(CH) => ~4.0  ms (F(CH) = 4 MHz)
  McoWaitExp115  = 11,  ///< 2^15 / F(CH) => ~8.0  ms (F(CH) = 4 MHz)
  McoWaitExp117  = 12,  ///< 2^17 / F(CH) => ~33.0 ms (F(CH) = 4 MHz)
  McoWaitExp119  = 13,  ///< 2^18 / F(CH) => ~131  ms (F(CH) = 4 MHz)
  McoWaitExp121  = 14,  ///< 2^21 / F(CH) => ~524  ms (F(CH) = 4 MHz)
  McoWaitExp123  = 15   ///< 2^23 / F(CH) => ~2.0  s  (F(CH) = 4 MHz)
} en_clk_mcowaittime_t;

/**
 ******************************************************************************
 ** \brief PLL Clock oscillation stabilization wait time
 ** 
 ** Enumeration for the PLL Clock oscillation stabilization wait time settings
 ******************************************************************************/
typedef enum en_clk_pllowaittime
{
  PlloWaitExp19   = 0,   ///< 2^9  / F(CH) => ~128  us (F(CH) = 4 MHz)
  PlloWaitExp110  = 1,   ///< 2^10 / F(CH) => ~256  us (F(CH) = 4 MHz)
  PlloWaitExp111  = 2,   ///< 2^11 / F(CH) => ~512  us (F(CH) = 4 MHz)
  PlloWaitExp112  = 3,   ///< 2^12 / F(CH) => ~1.02 ms (F(CH) = 4 MHz)
  PlloWaitExp113  = 4,   ///< 2^13 / F(CH) => ~2.05 ms (F(CH) = 4 MHz)
  PlloWaitExp114  = 5,   ///< 2^14 / F(CH) => ~4.10 ms (F(CH) = 4 MHz)
  PlloWaitExp115  = 6,   ///< 2^15 / F(CH) => ~8.20 ms (F(CH) = 4 MHz)
  PlloWaitExp116  = 7    ///< 2^16 / F(CH) => ~16.4 ms (F(CH) = 4 MHz)
} en_clk_pllowaittime_t;

/**
 ******************************************************************************
 ** \brief PLL SourceClock (PINC bit of PSW_TMR)
 ** 
 ** Enumeration for the PLL Clock Source.
 **
 ** \attention HS-RC source is only available if used device supports PLL-CLKHC
 **            setting! PINC bit availability is <b>not</b> checked in this
 **            driver!
 ******************************************************************************/
typedef enum en_clk_pll_src
{
  PllSrcClkMo = 0,   ///< Use Main Clock as PLL source (always available, default)
  PllSrcClkHc = 123  ///< Use HS-RC Clock as PLL source (only if available!)
} en_clk_pll_src_t;

/**
 ******************************************************************************
 ** \brief Datatype for holding internal data needed for CLK
 ******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_CLK == PDL_ON)
typedef struct stc_clk_intern_data
{
  func_ptr_t            pfnPllStabCb;     ///< PLL stabilization callback function pointer
  func_ptr_t            pfnScoStabCb;     ///< Sub Clock stabilization callback function pointer
  func_ptr_t            pfnMcoStabCb;     ///< Main Clock stabilization callback function pointer
} stc_clk_intern_data_t ;
#endif

/**
 ******************************************************************************
 ** \brief Clock Gate peripheral enumerators
 ******************************************************************************/
typedef enum en_clk_gate_peripheral
{
  ClkGateGpio   = 0,    ///< GPIO clock gate
  ClkGateDma    = 2,    ///< DMA clock gate
  ClkGateAdc0   = 3,    ///< ADC0 clock gate
  ClkGateAdc1   = 4,    ///< ADC1 clock gate
  ClkGateAdc2   = 5,    ///< ADC2 clock gate
  ClkGateAdc3   = 6,    ///< ADC3 clock gate
  ClkGateMfs0   = 7,    ///< MFS0 clock gate
  ClkGateMfs1   = 8,    ///< MFS1 clock gate
  ClkGateMfs2   = 9,    ///< MFS2 clock gate
  ClkGateMfs3   = 10,   ///< MFS3 clock gate
  ClkGateMfs4   = 11,   ///< MFS4 clock gate
  ClkGateMfs5   = 12,   ///< MFS5 clock gate
  ClkGateMfs6   = 13,   ///< MFS6 clock gate
  ClkGateMfs7   = 14,   ///< MFS7 clock gate
  ClkGateMfs8   = 15,   ///< MFS8 clock gate
  ClkGateMfs9   = 16,   ///< MFS9 clock gate
  ClkGateMfs10  = 17,   ///< MFS10 clock gate
  ClkGateMfs11  = 18,   ///< MFS11 clock gate
  ClkGateMfs12  = 19,   ///< MFS12 clock gate
  ClkGateMfs13  = 20,   ///< MFS13 clock gate
  ClkGateMfs14  = 21,   ///< MFS14 clock gate
  ClkGateMfs15  = 22,   ///< MFS15 clock gate
  ClkGateQprc0  = 23,   ///< QPRC0 clock gate
  ClkGateQprc1  = 24,   ///< QPRC1 clock gate
  ClkGateQprc2  = 25,   ///< QPRC2 clock gate
  ClkGateQprc3  = 26,   ///< QPRC3 clock gate
  ClkGateMft0   = 27,   ///< MFT0, PPG0/2/4/6 clock gate
  ClkGateMft1   = 28,   ///< MFT1, PPG8/10/12/14 clock gate
  ClkGateMft2   = 29,   ///< MFT2, PPG16/18/20/22 clock gate
  ClkGateMft3   = 30,   ///< MFT3, PPG24/26/28/30 clock gate
  ClkGateBt0    = 31,   ///< BT0/1/2/3 clock gate
  ClkGateBt4    = 32,   ///< BT4/5/6/7 clock gate
  ClkGateBt8    = 33,   ///< BT8/9/10/11 clock gate
  ClkGateBt12   = 34,   ///< BT12/13/14/15 clock gate
  ClkGateCan0   = 36,   ///< CAN0 clock gate
  ClkGateCan1   = 37,   ///< CAN1 clock gate
} en_clk_gate_peripheral_t;

/**
 ******************************************************************************
 ** \brief Reset peripheral enumerators
 ******************************************************************************/
typedef enum en_clk_reset_peripheral
{
  ClkResetDma    = 2,    ///< DMA Reset
  ClkResetAdc0   = 3,    ///< ADC0 Reset
  ClkResetAdc1   = 4,    ///< ADC1 Reset
  ClkResetAdc2   = 5,    ///< ADC2 Reset
  ClkResetAdc3   = 6,    ///< ADC3 Reset
  ClkResetMfs0   = 7,    ///< MFS0 Reset
  ClkResetMfs1   = 8,    ///< MFS1 Reset
  ClkResetMfs2   = 9,    ///< MFS2 Reset
  ClkResetMfs3   = 10,   ///< MFS3 Reset
  ClkResetMfs4   = 11,   ///< MFS4 Reset
  ClkResetMfs5   = 12,   ///< MFS5 Reset
  ClkResetMfs6   = 13,   ///< MFS6 Reset
  ClkResetMfs7   = 14,   ///< MFS7 Reset
  ClkResetMfs8   = 15,   ///< MFS8 Reset
  ClkResetMfs9   = 16,   ///< MFS9 Reset
  ClkResetMfs10  = 17,   ///< MFS10 Reset
  ClkResetMfs11  = 18,   ///< MFS11 Reset
  ClkResetMfs12  = 19,   ///< MFS12 Reset
  ClkResetMfs13  = 20,   ///< MFS13 Reset
  ClkResetMfs14  = 21,   ///< MFS14 Reset
  ClkResetMfs15  = 22,   ///< MFS15 Reset
  ClkResetQprc0  = 23,   ///< QPRC0 Reset
  ClkResetQprc1  = 24,   ///< QPRC1 Reset
  ClkResetQprc2  = 25,   ///< QPRC2 Reset
  ClkResetQprc3  = 26,   ///< QPRC3 Reset
  ClkResetMft0   = 27,   ///< MFT0, PPG0/2/4/6 Reset
  ClkResetMft1   = 28,   ///< MFT1, PPG8/10/12/14 Reset
  ClkResetMft2   = 29,   ///< MFT2, PPG16/18/20/22 Reset
  ClkResetMft3   = 30,   ///< MFT3, PPG24/26/28/30 Reset
  ClkResetBt0    = 31,   ///< BT0/1/2/3 Reset
  ClkResetBt4    = 32,   ///< BT4/5/6/7 Reset
  ClkResetBt8    = 33,   ///< BT8/9/19/11 Reset
  ClkResetBt12   = 34,   ///< BT12/13/14/15 Reset
  ClkResetSdIf   = 35,   ///< SD Card I/F Reset
  ClkResetCan0   = 36,   ///< CAN0 Reset
  ClkResetCan1   = 37,   ///< CAN1 Reset
} en_clk_reset_peripheral_t;

/**
 ******************************************************************************
 ** \brief Clock configuration
 ** 
 ** The Clock configuration settings
 ******************************************************************************/
typedef struct stc_clk_config
{   
  en_clk_baseclkdiv_t   enBaseClkDiv;     ///< See description of #en_clk_baseclkdiv_t
  en_clk_apb0div_t      enAPB0Div;        ///< See description of #en_clk_apb0div_t
  en_clk_apb1div_t      enAPB1Div;        ///< See description of #en_clk_apb1div_t
  boolean_t             bAPB1Disable;     ///< TRUE: Disables APB1 regardless of divider settings
  en_clk_mcowaittime_t  enMCOWaitTime;    ///< See description of #en_clk_mcowaittime_t
  en_clk_scowaittime_t  enSCOWaitTime;    ///< See description of #en_clk_scowaittime_t
  en_clk_pllowaittime_t enPLLOWaitTime;   ///< See description of #en_clk_pllowaittime_t
  uint8_t               u8PllK;           ///< PLL input clock frequency division ratio, PLLK
  uint8_t               u8PllM;           ///< PLL VCO clock frequency division ratio, PLLM
  uint8_t               u8PllN;           ///< PLL feedback frequency division ration, PLLN
#if (PDL_INTERRUPT_ENABLE_CLK == PDL_ON)
  boolean_t             bPllIrq;          ///< TRUE: enable PLL oscillation stabilization completion interrupt
  boolean_t             bMcoIrq;          ///< TRUE: enable Main Clock oscillation stabilization completion interrupt
  boolean_t             bScoIrq;          ///< TRUE: enable Sub Clock oscillation stabilization completion interrupt
  func_ptr_t            pfnPllStabCb;     ///< PLL stabilization callback function pointer
  func_ptr_t            pfnMcoStabCb;     ///< Main Clock stabilization callback function pointer
  func_ptr_t            pfnScoStabCb;     ///< Sub Clock stabilization callback function pointer
#endif   
} stc_clk_config_t;

/******************************************************************************/
/* Global variable definitions ('extern')                                     */
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_CLK == PDL_ON)
extern stc_clk_intern_data_t stcClkInternData;
#endif

/******************************************************************************/
/* Global function prototypes (definition in C source)                        */
/******************************************************************************/

#if (PDL_INTERRUPT_ENABLE_CLK == PDL_ON)
void Clk_IrqHandler(void) ;
#endif
/* Set clock divider, wait time, interrupt */
en_result_t Clk_Init(stc_clk_config_t* pstcClk) ;
/* Clock enable/disable */
en_result_t Clk_EnableHscr(boolean_t bBlock);
en_result_t Clk_DisableHscr(void);
en_result_t Clk_EnableMainClock(boolean_t bBlock);
en_result_t Clk_DisableMainClock(void);
en_result_t Clk_EnableSubClock(boolean_t bBlock);
en_result_t Clk_DisableSubClock(void);
en_result_t Clk_EnablePllClock(boolean_t bBlock);
en_result_t Clk_DisablePllClock(void);
/* Set the source clock */
en_result_t Clk_SetSource(en_clk_source_t enSource);
/* Peripheral clock enable/disable/status read */
en_result_t Clk_PeripheralClockEnable(en_clk_gate_peripheral_t enPeripheral) ;
en_result_t Clk_PeripheralClockDisable(en_clk_gate_peripheral_t enPeripheral) ;
boolean_t Clk_PeripheralGetClockState(en_clk_gate_peripheral_t enPeripheral) ;
/* Peripheral set reset/release reset */
en_result_t Clk_PeripheralSetReset(en_clk_reset_peripheral_t enPeripheral) ;
en_result_t Clk_PeripheralClearReset(en_clk_reset_peripheral_t enPeripheral) ;

//@} // ClkGroup

#ifdef __cplusplus
}
#endif

#endif // #if (defined(PDL_PERIPHERAL_CLK_ACTIVE))

#endif /* __CLK_H__ */
/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
