/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file dma.c
 **
 ** A detailed description is available at 
 ** @link DmaGroup DMA description @endlink
 **
 ** History:
 **   - 2014-02-19  0.1  Edison Zhang  First version
 **   - 2014-03-06  0.2  Edison Zhang  Update Dma_DeInit_Channel() function
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "dma.h"

#if (defined(PDL_PERIPHERAL_DMA_ACTIVE))

/**
 ******************************************************************************
 ** \addtogroup DmaGroup
 ******************************************************************************/
//@{

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/
// DMACAn => FM3_DMAC_BASE + size of register set + offset of DMACA0
// DMACAn => 0x40060000    + 0x10 * n             + 0x10
#define GET_DMA_ADDRESS(x) (FM0P_DMAC_BASE + (0x10ul * (x)) + 0x10ul)

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_DMA0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_DMA1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_DMA2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_DMA3 == PDL_ON)    
static stc_dma_intern_data_t stcDmaInternData; ///< Store DMA internal data  
#endif

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/
typedef union
{
	uint32_t                 u32DMACA;
	stc_dmac_dmaca0_field_t  stcDMACA;
	
}un_dmac_dmaca_t;
  
typedef union
{
	stc_dmac_dmacb0_field_t  stcDMACB;
	uint32_t                 u32DMACB;
	
}un_dmac_dmacb_t;

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_DMA0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_DMA1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_DMA2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_DMA3 == PDL_ON) 
/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/
static en_result_t Dma_Irq_Init(uint8_t u8DmaChannel);
static en_result_t Dma_Irq_DeInit(uint8_t u8DmaChannel);
#endif

#if (PDL_INTERRUPT_ENABLE_DMA0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_DMA1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_DMA2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_DMA3 == PDL_ON) 
/**
 ******************************************************************************
 ** \brief DMA interrupt handler
 **
 ** \param [in]  u8DmaChannel     Channel number 
 **
 ** \note depending on the DMA stop status the DmaCallback function is called
 **       on successful transfer or DmaErrorCallback function on errornous
 **       transfer. The ErrorCallback has the error code as an (uint8_t)
 **       argument which reflects the 3 bits of the stop status.
 ******************************************************************************/
void DmaIrqHandler(uint8_t u8DmaChannel)
{
  uint32_t u32DmaRegisterBaseAddress;
  uint32_t u32RegisterReadOut;
  uint8_t  u8StopStatus;
  
  u32DmaRegisterBaseAddress = GET_DMA_ADDRESS(u8DmaChannel);
  
  // Readout DMACB and update
  u32RegisterReadOut = *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMACB_OFFSET));
  
  u8StopStatus = (uint8_t)((u32RegisterReadOut & 0x00070000ul)>> 16u);
  
  // Clear interrupt cause
  u32RegisterReadOut &= 0xFFF8FFFF;     
  *(uint32_t*)(uint32_t)((u32DmaRegisterBaseAddress + DMA_DMACB_OFFSET)) 
    = u32RegisterReadOut;  
  
  if (0x5u == u8StopStatus)  // Successful DMA transfer
  {
    stcDmaInternData.apfnDmaCallback[u8DmaChannel](); 
  }
  else                      // Errornous DMA transfer
  {
    stcDmaInternData.apfnDmaErrorCallback[u8DmaChannel](u8StopStatus);
  }
}

/**
 ******************************************************************************
 ** \brief Init DMA interrupt IRQ
 **
 ** \param [in]  u8DmaChannel     Channel number 
 **
 ** \retval Ok                    init successful
 **
 ******************************************************************************/
static en_result_t Dma_Irq_Init(uint8_t u8DmaChannel)
{
  switch(u8DmaChannel)
  {
    case 0u:
      NVIC_ClearPendingIRQ(MFS6_14_RX_DMA0_IRQn);
      NVIC_EnableIRQ(MFS6_14_RX_DMA0_IRQn);
      NVIC_SetPriority(MFS6_14_RX_DMA0_IRQn, PDL_IRQ_LEVEL_MFS6_14_RX_DMA0);    
    break;
    case 1u:
      NVIC_ClearPendingIRQ(MFS6_14_TX_DMA1_IRQn);
      NVIC_EnableIRQ(MFS6_14_TX_DMA1_IRQn);
      NVIC_SetPriority(MFS6_14_TX_DMA1_IRQn, PDL_IRQ_LEVEL_MFS6_14_TX_DMA1);    
    break;
    case 2u:
      NVIC_ClearPendingIRQ(MFS7_15_RX_DMA2_IRQn);
      NVIC_EnableIRQ(MFS7_15_RX_DMA2_IRQn);
      NVIC_SetPriority(MFS7_15_RX_DMA2_IRQn, PDL_IRQ_LEVEL_MFS7_15_RX_DMA2);    
    break;
    case 3u:
      NVIC_ClearPendingIRQ(MFS7_15_TX_DMA3_IRQn);
      NVIC_EnableIRQ(MFS7_15_TX_DMA3_IRQn);
      NVIC_SetPriority(MFS7_15_TX_DMA3_IRQn, PDL_IRQ_LEVEL_MFS7_15_TX_DMA3);    
    break;  
    default:
    break;
  }
  return Ok;
}

/**
 ******************************************************************************
 ** \brief De-Init DMA interrupt
 **
 ** \param [in]  u8DmaChannel     Channel number 
 **
 ** \retval Ok                    de-init successful
 **
 ******************************************************************************/
static en_result_t Dma_Irq_DeInit(uint8_t u8DmaChannel)
{
  switch(u8DmaChannel)
  {
    case 0:
#if (PDL_INTERRUPT_ENABLE_MFS6 == PDL_OFF) && (PDL_INTERRUPT_ENABLE_MFS14 == PDL_OFF)        
      NVIC_ClearPendingIRQ(MFS6_14_RX_DMA0_IRQn);
      NVIC_DisableIRQ(MFS6_14_RX_DMA0_IRQn);
      NVIC_SetPriority(MFS6_14_RX_DMA0_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);   
#endif      
    break;
    case 1:
#if (PDL_INTERRUPT_ENABLE_MFS6 == PDL_OFF) && (PDL_INTERRUPT_ENABLE_MFS14 == PDL_OFF)       
      NVIC_ClearPendingIRQ(MFS6_14_TX_DMA1_IRQn);
      NVIC_DisableIRQ(MFS6_14_TX_DMA1_IRQn);
      NVIC_SetPriority(MFS6_14_TX_DMA1_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);    
#endif      
    break;
    case 2:
#if (PDL_INTERRUPT_ENABLE_MFS7 == PDL_OFF) && (PDL_INTERRUPT_ENABLE_MFS15 == PDL_OFF)        
      NVIC_ClearPendingIRQ(MFS7_15_RX_DMA2_IRQn);
      NVIC_DisableIRQ(MFS7_15_RX_DMA2_IRQn);
      NVIC_SetPriority(MFS7_15_RX_DMA2_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);   
#endif      
    break;
    case 3:
#if (PDL_INTERRUPT_ENABLE_MFS7 == PDL_OFF) && (PDL_INTERRUPT_ENABLE_MFS15 == PDL_OFF)         
      NVIC_ClearPendingIRQ(MFS7_15_TX_DMA3_IRQn);
      NVIC_DisableIRQ(MFS7_15_TX_DMA3_IRQn);
      NVIC_SetPriority(MFS7_15_TX_DMA3_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);   
#endif      
    break;   
    default:
    break;
  }
  return Ok;
}

/**
 ******************************************************************************
 ** \brief Enable interrupt of a DMA channel
 **
 ** Enable interrupt and set interrupt callback function of a DMA channel
 **
 ** \param [in]  u8DmaChannel    DMA channel index
 ** \param [in]  pstcIntSel      DMA interrupt selection structure
 ** \param [in]  pstcIntCb       DMA interrupt callback function
 **
 ** \retval Ok                    Interrupt enabled normally
 ** \retval ErrorInvalidParameter If one of followng cases matchs:
 **                               - u8DmaChannel > DMA_MAX_CH_INDEX 
 **                               - Other invalid configuration
 **
 ******************************************************************************/
en_result_t Dma_EnableInt(uint8_t u8DmaChannel, 
                          stc_dma_int_sel_t* pstcIntSel,
                          stc_dma_int_cb_t* pstcIntCb  )
{
    uint32_t u32DmaRegisterBaseAddress; 
    un_dmac_dmacb_t unDmacDmacb;
    
    if(u8DmaChannel > DMA_MAX_CH_INDEX)
    {
        return ErrorInvalidParameter;
    }
    
    u32DmaRegisterBaseAddress = GET_DMA_ADDRESS(u8DmaChannel);
    
    // Readout original DMACB
    unDmacDmacb.u32DMACB = *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMACB_OFFSET)) ;
    
    // Update EI and CI bit
    if(pstcIntSel->bCompleteInt == TRUE)
    {
        stcDmaInternData.apfnDmaCallback[u8DmaChannel] = pstcIntCb->pfnCompleteCallback;
        unDmacDmacb.stcDMACB.CI = 1;
    }
    
    if(pstcIntSel->bErrorInt == TRUE)
    {
        stcDmaInternData.apfnDmaErrorCallback[u8DmaChannel] = pstcIntCb->pfnErrorCallback;
        unDmacDmacb.stcDMACB.EI = 1;
    }
    
    // Write back to DMACB
    *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMACB_OFFSET)) = unDmacDmacb.u32DMACB;
    
    Dma_Irq_Init(u8DmaChannel);
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable interrupt of a DMA channel
 **
 ** \param [in]  u8DmaChannel    DMA channel index
 ** \param [in]  pstcIntSel      DMA interrupt selection structure
 **
 ** \retval Ok                    Interrupt disabled normally
 ** \retval ErrorInvalidParameter If one of following cases matchs:
 **                               - u8DmaChannel > DMA_MAX_CH_INDEX 
 **                               - Other invalid configuration
 **
 ******************************************************************************/
en_result_t Dma_DisableInt(uint8_t u8DmaChannel, 
                           stc_dma_int_sel_t* pstcIntSel )
{
    uint32_t u32DmaRegisterBaseAddress;
    un_dmac_dmacb_t unDmacDmacb;
    
    if(u8DmaChannel > DMA_MAX_CH_INDEX)
    {
        return ErrorInvalidParameter;
    }
    
    u32DmaRegisterBaseAddress = GET_DMA_ADDRESS(u8DmaChannel);
    
    // Readout original DMACB
    unDmacDmacb.u32DMACB = *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMACB_OFFSET)) ;
    
    // Update EI and CI bit
    if(pstcIntSel->bCompleteInt == TRUE)
    {
        stcDmaInternData.apfnDmaCallback[u8DmaChannel] = NULL;
        unDmacDmacb.stcDMACB.CI = 0;
    }
    
    if(pstcIntSel->bErrorInt == TRUE)
    {
        stcDmaInternData.apfnDmaErrorCallback[u8DmaChannel] = NULL;
        unDmacDmacb.stcDMACB.EI = 0;
    }
    
    // Write back to DMACB
    *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMACB_OFFSET)) = unDmacDmacb.u32DMACB;
    
    Dma_Irq_DeInit(u8DmaChannel);
    
    return Ok;
}

#endif

/**
 ******************************************************************************
 ** \brief Initializes a DMA channel
 **
 ** Sets up an DMA channel without starting immediate DMA transfer.
 ** Set_Dma_Channel() is used for starting DMA transfer.
 **
 ** \param [in]  u8DmaChannel     DMA channel number
 ** \param [in]  pstcConfig       DMA module configuration 
 **
 ** \retval Ok                    init successful
 ** \retval ErrorInvalidParameter pstcAdc == NULL or other invalid configuration
 ** \retval OperationInProgress   DMA channel already in use
 **
 ******************************************************************************/
en_result_t Dma_Init_Channel(uint8_t u8DmaChannel, volatile stc_dma_config_t* pstcConfig)                   
{  
  un_dmac_dmaca_t unDmacDmaca;
  un_dmac_dmacb_t unDmacDmacb;
  uint32_t        u32DmaRegisterBaseAddress;
  uint32_t u32DrqselBit;

  // Check for NULL pointer
  if (pstcConfig == NULL)
  {
    return ErrorInvalidParameter ;
  }
  
  unDmacDmaca.u32DMACA = 0u;
  unDmacDmacb.u32DMACB = 0u;

  u32DmaRegisterBaseAddress = GET_DMA_ADDRESS(u8DmaChannel);
  
  // Setup DMACA ...
  //
  // Block Count
  unDmacDmaca.stcDMACA.BC = 0x0Fu & (uint32_t)(pstcConfig->u8BlockCount - 1);
  
  // Transfer Count
  unDmacDmaca.stcDMACA.TC = 0xFFFFu & (uint32_t)(pstcConfig->u16TransferCount - 1);
  
  // Interrupt Request Channel
  if (pstcConfig->enDmaIdrq == Software)
  {
    unDmacDmaca.stcDMACA.IS = 0u;
  }
  else
  {
    unDmacDmaca.stcDMACA.IS = 0x20u | (0x1Fu &  (uint32_t)(pstcConfig->enDmaIdrq));
  }
  
  // ... and update hardware
  *(uint32_t*)(uint32_t)((u32DmaRegisterBaseAddress + DMA_DMACA_OFFSET)) = unDmacDmaca.u32DMACA ;
  
  // Setup DMACB ...
  //
  // First Transfer mode
  switch (pstcConfig->enTransferMode)
  {
    case DmaBlockTransfer:
      unDmacDmacb.stcDMACB.MS = 0u;
      break;
    case DmaBurstTransfer:
      unDmacDmacb.stcDMACB.MS = 1u;
      break;
    case DmaDemandTransfer:
      unDmacDmacb.stcDMACB.MS = 2u;
      break;
    default :
      return ErrorInvalidParameter;
  }
  
  // Transfer width
  switch (pstcConfig->enTransferWdith)
  {
    case Dma8Bit:
      unDmacDmacb.stcDMACB.TW = 0u;
      break;
    case Dma16Bit:
      unDmacDmacb.stcDMACB.TW = 1u;
      break;
    case Dma32Bit:
      unDmacDmacb.stcDMACB.TW = 2u;
      break;
    default :
      return ErrorInvalidParameter;
  }
 
  // Fixed source
  unDmacDmacb.stcDMACB.FS = (uint8_t)((pstcConfig->bFixedSource == TRUE) ? 1u : 0u);
  
  // Fixed destination
  unDmacDmacb.stcDMACB.FD = (uint8_t)((pstcConfig->bFixedDestination == TRUE) ? 1u : 0u);
  
  // Reload Count (BC/TC reload)
  unDmacDmacb.stcDMACB.RC = (uint8_t)((pstcConfig->bReloadCount == TRUE) ? 1u : 0u);
  
  // Reload Source
  unDmacDmacb.stcDMACB.RS = (uint8_t)((pstcConfig->bReloadSource == TRUE) ? 1u : 0u);
  
  // Reload Destination
  unDmacDmacb.stcDMACB.RD = (uint8_t)((pstcConfig->bReloadDestination == TRUE) ? 1u : 0u);
  
  // Enable bit mask
  unDmacDmacb.stcDMACB.EM = (uint8_t)((pstcConfig->bEnableBitMask == TRUE) ? 1u : 0u);
  
  // ... and update hardware
  *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMACB_OFFSET))
    = unDmacDmacb.u32DMACB ;
  
  // Setup source address
  *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMASA_OFFSET))
    = pstcConfig->u32SourceAddress;
  
  // Setup destination address
  *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMADA_OFFSET))
    = pstcConfig->u32DestinationAddress;
  
  // Switch resource interrupt to DMA (except software DMA)
  if (pstcConfig->enDmaIdrq != Software)
  {
    u32DrqselBit = 1ul << (uint32_t)(pstcConfig->enDmaIdrq);
    FM0P_INTREQ->DRQSEL |= u32DrqselBit;
  }
  
  return Ok;
}

/**
 ******************************************************************************
 ** \brief De-Initializes a DMA channel
 **
 ** Clears an DMA channel.
 **
 ** \param [in]  u8DmaChannel     DMA channel number
 **
 ** \retval Ok                    init successful
 ** \retval ErrorInvalidParameter pstcAdc == NULL or other invalid configuration
 **
 ******************************************************************************/
en_result_t Dma_DeInit_Channel(uint8_t u8DmaChannel)  
{
  uint32_t u32DmaRegisterBaseAddress;
  
  Dma_Irq_DeInit(u8DmaChannel);
  
  u32DmaRegisterBaseAddress = GET_DMA_ADDRESS(u8DmaChannel);
  
  // clear all registers of DMA channel
  *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMACA_OFFSET)) = 0u;
  *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMACB_OFFSET)) = 0u;
  *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMASA_OFFSET)) = 0u;
  *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMADA_OFFSET)) = 0u;
  
  return Ok;
} // Dma_DeInit_Channel

/**
 ******************************************************************************
 ** \brief Enable, disable, pause, or trigger a DMA channel via configuration
 **
 ** This function enables, disables, pauses or triggers a DMA transfer according
 ** to the settings in the configuration bits for EB (Enable), PB (Pause) and
 ** ST (Software Trigger)
 **
 ** \param [in]  u8DmaChannel     DMA channel number
 ** \param [in]  bEnable          Enable the channel or not
 ** \param [in]  bPause           Pause the channel or not
 ** \param [in]  bSoftwareTrigger Trigger DMA by software or not
 **
 ** \retval Ok                    Setting finished
 **
 ******************************************************************************/
en_result_t Dma_SetChannel (uint8_t u8DmaChannel, 
                            boolean_t bEnable,
                            boolean_t bPause,
                            boolean_t bSoftwareTrigger) 
{
  un_dmac_dmaca_t unDmacDmaca;
  uint32_t u32RegisterReadOut ;
  uint32_t u32DmaRegisterBaseAddress ;

  unDmacDmaca.u32DMACA = 0u;
  unDmacDmaca.stcDMACA.EB = (uint8_t)((bEnable == TRUE) ? 1u : 0u);
  unDmacDmaca.stcDMACA.PB = (uint8_t)((bPause == TRUE) ? 1u : 0u);
  unDmacDmaca.stcDMACA.ST = (uint8_t)((bSoftwareTrigger == TRUE) ? 1u : 0u);
  
  u32DmaRegisterBaseAddress = GET_DMA_ADDRESS(u8DmaChannel);
  
  // Readout DMACA and update
  u32RegisterReadOut = *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMACA_OFFSET)) ;
  unDmacDmaca.u32DMACA |= (u32RegisterReadOut & 0x1FFFFFFFu);   // masking EB, PB, ST ...
  *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMACA_OFFSET)) = unDmacDmaca.u32DMACA ;
  
  return Ok;
}

/**
 ******************************************************************************
 ** \brief Read stop cause of a DMA channel
 **
 ** \param [in]  u8DmaChannel       DMA channel
 **
 ** \retval DmaStopByAddrOverflow      DMA stops by address overflow
 ** \retval DmaStopByTransferStopRqst  DMA stops by transfer stop request
 ** \retval DmaStopBySrcAccessError    DMA stops by source access error
 ** \retval DmaStopByDstAccessError    DMA stops by destination access error
 ** \retval DmaStopByTransferComplete  DMA stops by transfer completion
 ** \retval DmaStopByTransferPause     DMA stops by transfer pause
 **
 ******************************************************************************/
en_dma_stop_cause_t Dma_GetStopCause(uint8_t u8DmaChannel)
{
    un_dmac_dmacb_t unDmacDmacb;
    uint32_t u32DmaRegisterBaseAddress;
    
    u32DmaRegisterBaseAddress = GET_DMA_ADDRESS(u8DmaChannel);
   
    // Readout original DMACB
    unDmacDmacb.u32DMACB = *(uint32_t*)((uint32_t)(u32DmaRegisterBaseAddress + DMA_DMACB_OFFSET)) ;
    
    return (en_dma_stop_cause_t) (unDmacDmacb.stcDMACB.SS);
}

/**
 ******************************************************************************
 ** \brief Enable DMA globally
 **
 ** \retval Ok                    Enable finished
 **
 ******************************************************************************/
en_result_t Dma_Enable(void)
{
  FM0P_DMAC->DMACR = 0x80000000ul; /// Set DE Bit (DMA enable all channels)
  
  return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable DMA globally
 **
 ** \retval Ok                    Disable finished
 **
 ******************************************************************************/
en_result_t Dma_Disable(void)
{
  FM0P_DMAC->DMACR = 0u; /// Clear everything
  
  return Ok;
}


//@} // DmaGroup

#endif // #if (defined(PDL_PERIPHERAL_DMA_ACTIVE))

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/

