/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/

/******************************************************************************/
/** \file flash.c
 **
 ** A detailed description is available at 
 ** @link MainFlashGroup Main Flash Module description @endlink
 **
 ** History:
 **   - 2014-02-21  0.1  Edison Zhang  First version.
 **
 ******************************************************************************/
/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "flash.h"

#if defined(PDL_PERIPHERAL_FLASH_ACTIVE) 

/*!
 ******************************************************************************
 ** \brief Read a half word data from Flash
 ** \param addr Pointer to read data address
 ******************************************************************************
 */
#define Flash_Read(addr)        *(volatile uint16_t*)((uint32_t)(addr))

/*!
 ******************************************************************************
 ** \brief Wirte a half word data into Flash
 ** \param addr Pointer to read data address
 ** \param data Write data
 ******************************************************************************
 */
#define Flash_Write(addr, data) *(volatile uint16_t*)((uint32_t)(addr)) = ( uint16_t)(data)

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/   
#if defined ( __ICCARM__ )
__ramfunc 
#endif
static uint8_t MFlash_CheckToggle( uint16_t* pAddr );

#if defined ( __ICCARM__ )
__ramfunc 
#endif 
static void MFlash_ReadResetCmd(uint16_t* pResetSecAddr);

/*!
 ******************************************************************************
 ** \brief Issue read/reset command
 **
 ** \param pu16ResetSecAddr address of reset sector  
 **
 ** \return None
 **
 ******************************************************************************
 */
#if defined ( __ICCARM__ )
__ramfunc 
#endif 
static void MFlash_ReadResetCmd(uint16_t* pu16ResetSecAddr)
{
    uint8_t  u8Dummy;

    /*  issue read/reset command    */
    Flash_Write(0x0000, 0xF0U) ;
    u8Dummy = Flash_Read(pu16ResetSecAddr) ;
    if(u8Dummy)  /* avoid warning */  
        ;
    return ;
}

/*!
 ******************************************************************************
 ** \brief Flash chip erase       
 **
 ** \param bCrRemain CR remain flag  
 **
 ** \retval Ok                       Flash chip erase normally
 ** \retval ErrorInvalidParameter    bCrRemain > 1
 ** \retval Error                    Flash erase failure
 **
 ******************************************************************************
 */
#if defined ( __ICCARM__ )
__ramfunc 
#endif 
en_result_t MFlash_ChipErase(boolean_t bCrRemain)
{
    uint8_t  u8Cnt;
    uint32_t u32CrData, u32CrAddr;
    uint16_t u16WriteData;
    en_result_t enRetValue = Ok;
    if(bCrRemain > 1)
    {
        enRetValue = ErrorInvalidParameter;
    }
    else
    {
        __disable_irq(); 
        /* Save CR value */
        if (bCrRemain == TRUE)
        {
            u32CrData = *(uint32_t*)CR_DATA_ADDR;
        }

        Flash_Write(MFLASH_CODE1, 0x00AA);
        Flash_Write(MFLASH_CODE2, 0x0055);
        Flash_Write(MFLASH_CODE1, 0x0080);
        Flash_Write(MFLASH_CODE1, 0x00AA);
        Flash_Write(MFLASH_CODE2, 0x0055);
        Flash_Write(MFLASH_CODE1, 0x0010);

         /*  if execution result of the automatic algorithm of flash memory is abnormally completed  */
        if( MFlash_CheckToggle((uint16_t*)0) == MFLASH_CHK_TOGG_ABNORMAL)
        {
            /*  sending the read/reset command to the reset sector  */
            MFlash_ReadResetCmd((uint16_t*)0) ;

            /*  return flash operation abnormally   */
            enRetValue  = Error ;
        }

        /* restore CR data in Flash */
        if (bCrRemain == TRUE)
        {
            for(u8Cnt=2,u16WriteData=(uint16_t)u32CrData, u32CrAddr = CR_DATA_ADDR;u8Cnt;u8Cnt--)
            {
                /*  issue write command   */
                Flash_Write(MFLASH_CODE1, 0x00AA) ;
                Flash_Write(MFLASH_CODE2, 0x0055) ;
                Flash_Write(MFLASH_CODE1, 0x00A0) ;
                Flash_Write((uint16_t*)u32CrAddr, u16WriteData);
                
                /*  execution result of the automatic algorithm of flash memory is abnormally complete or verify error  */
                if(( MFlash_CheckToggle((uint16_t*)u32CrAddr) == MFLASH_CHK_TOGG_ABNORMAL) ||
                  ( Flash_Read((uint16_t*)u32CrAddr) != u16WriteData))
                {
                    /*  issue read/reset command to reset sector    */
                    MFlash_ReadResetCmd((uint16_t*)u32CrAddr) ;
        
                    /*  return flash operation abnormally   */
                    enRetValue  = Error ;
                    break;
                }      
                u16WriteData = (uint16_t)(u32CrData>>16);
                u32CrAddr += 2;
            }
        }
               
        /* Recover IRQ  */
        __enable_irq();
    }
    return enRetValue;
}

/*!
 ******************************************************************************
 ** \brief Flash sector erase       
 **
 ** \param pu16SecAddr address of flash sector  
 **
 ** \retval Ok      Flash sector erase successfully
 ** \retval Error   Flash sector erase failure
 **
 ******************************************************************************
 */
#if defined ( __ICCARM__ )
__ramfunc 
#endif 
en_result_t MFlash_SectorErase(uint16_t* pu16SecAddr)
{
    en_result_t enRet = Ok;

    __disable_irq(); 
    Flash_Write(MFLASH_CODE1, 0x00AA);
    Flash_Write(MFLASH_CODE2, 0x0055);
    Flash_Write(MFLASH_CODE1, 0x0080);
    Flash_Write(MFLASH_CODE1, 0x00AA);
    Flash_Write(MFLASH_CODE2, 0x0055);
    Flash_Write(pu16SecAddr, 0x0030);

     /*  if execution result of the automatic algorithm of flash memory is abnormally completed  */
    if( MFlash_CheckToggle(pu16SecAddr) == MFLASH_CHK_TOGG_ABNORMAL)
    {
        /*  sending the read/reset command to the reset sector  */
        MFlash_ReadResetCmd(pu16SecAddr) ;

        /*  return flash operation abnormally   */
        enRet  = Error ;
    }

    /* Recover IRQ  */
    __enable_irq();
        
    return enRet;
}

/*!
 ******************************************************************************
 ** \brief Flash half-word write
 **
 ** \param pu16WriteAddr address of flash data
 ** \param pu16WriteData pointer to write data
 ** \param u32Size data size, 1 indicates 1 16-bit data
 **
 ** \retval Ok         Flash half-word write normally
 ** \retval Error      Flash half-word write failure
 **
 ******************************************************************************
 */
#if defined ( __ICCARM__ )
__ramfunc 
#endif 
en_result_t MFlash_Write(uint16_t*  pu16WriteAddr, 
                         uint16_t* pu16WriteData, 
                         uint32_t u32Size)
{
    en_result_t  enRetValue  = Ok ;
    uint32_t   u32Cnt;
    
    __disable_irq(); 
  
    for(u32Cnt=u32Size;u32Cnt;u32Cnt--)
    {
        /*  issue write command   */
        Flash_Write(MFLASH_CODE1, 0x00AA) ;
        Flash_Write(MFLASH_CODE2, 0x0055) ;
        Flash_Write(MFLASH_CODE1, 0x00A0) ;
        Flash_Write(pu16WriteAddr, (uint16_t)*pu16WriteData);
        
        /*  execution result of the automatic algorithm of flash memory is abnormally complete or verify error  */
        if(( MFlash_CheckToggle(pu16WriteAddr) == MFLASH_CHK_TOGG_ABNORMAL) ||
          ( Flash_Read(pu16WriteAddr) != *pu16WriteData))
        {
            /*  issue read/reset command to reset sector    */
            MFlash_ReadResetCmd(pu16WriteAddr) ;

            /*  return flash operation abnormally   */
            enRetValue  = Error ;
        }      
        /* Prepare next h-word write */
        pu16WriteAddr++;
        pu16WriteData++; 
    }
    
    /* Recover IRQ  */
    __enable_irq();


    return enRetValue ;
}

/*!
 ******************************************************************************
 ** \brief automatic algorithm of flash memory execution 
 **
 ** \param pu16Addr address of flash data
 **
 ** \retval MFLASH_RET_OK
 ** \retval MFLASH_RET_ABNORMAL
 ** \retval MFLASH_RET_INVALID_PARA
 **
 ******************************************************************************
 */
#if defined ( __ICCARM__ )
__ramfunc 
#endif 
static uint8_t MFlash_CheckToggle( uint16_t* pu16Addr )
{
    uint16_t   u16SequenceFlag1, u16SequenceFlag2 ;  /*  hardware sequence flag */
    uint8_t   u8RetValue  = MFLASH_CHK_TOGG_NORMAL ;

    /* set hardware sequence flag */
    u16SequenceFlag1 = Flash_Read(pu16Addr) ;
    u16SequenceFlag2 = Flash_Read(pu16Addr) ;
    /*  if automatic algorithm is executing */
    while((( u16SequenceFlag1 ^ u16SequenceFlag2) & MFLASH_CHK_TOGG_MASK) == MFLASH_CHK_TOGG_MASK)
    {
        /*  if exceeds the timing limit */
        if(( u16SequenceFlag1 & MFLASH_CHK_TLOV_MASK) == MFLASH_CHK_TLOV_MASK)
        {
            /* set hardware sequence flag */
            u16SequenceFlag1 = Flash_Read(pu16Addr) ;
            u16SequenceFlag2 = Flash_Read(pu16Addr) ;

            /*  if automatic algorithm is executing */
            if((( u16SequenceFlag1 ^ u16SequenceFlag2) & MFLASH_CHK_TOGG_MASK) == MFLASH_CHK_TOGG_MASK)
            {
                /*  abnormally complete */
                u8RetValue  = MFLASH_CHK_TOGG_ABNORMAL ;

                break;
            }
        }

        /* set hardware sequence flag */
        u16SequenceFlag1 = Flash_Read(pu16Addr) ;
        u16SequenceFlag2 = Flash_Read(pu16Addr) ;
    }

    return u8RetValue ;
}

#endif
