/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file lpm.c
 **
 ** A detailed description is available at 
 ** @link LpmGroup Low Power Modes Module description @endlink
 **
 ** History:
 **   - 2014-01-23  1.0  Edison Zhang  First version.
 **   - 2014-03-10  1.1  Edison Zhang  Update Lpm_ConfigDeepStbRetCause() function
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "lpm.h"

#if (defined(PDL_PERIPHERAL_LPM_ACTIVE))

/**
 ******************************************************************************
 ** \addtogroup LpmGroup
 ******************************************************************************/
//@{

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/
// SCR address
#define CM0P_CORE_CSR  *((volatile unsigned int*)(0xE000ED10UL))
   
// Backup registers base address   
#define LPCM_BACKUP_REG_BASE   ((volatile uint8_t*)0x40035900)
   
/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global functions                                                          */
/*---------------------------------------------------------------------------*/

/*!
 ******************************************************************************
 ** \brief WRFSR dummy read
 ******************************************************************************
 */
static uint16_t WRFSRDummyRead(void)
{
    uint16_t Dummy;
    Dummy = FM0P_DS->WRFSR;
    return Dummy;
}

/*!
 ******************************************************************************
 ** \brief WRFSR dummy read
 ******************************************************************************
 */
static uint16_t WIFSRDummyRead(void)
{
    uint16_t Dummy;
    Dummy = FM0P_DS->WIFSR;
    return Dummy;
}


/*!
 ******************************************************************************
 ** \brief go to standby mode
 **
 ** \param enMode the type of standby mode.
 **        This parameter can be one of the following values:
 ** \arg   StbSleepMode    Sleep mode
 ** \arg   StbTimerMode    Timer mode
 ** \arg   StbStopMode     Stop mode
 ** \arg   StbRtcMode      RTC mode
 ** \arg   DeepStbRtcMode  deep standby RTC mode
 ** \arg   DeepStbStopMode deep standby stop mode

 ** \param bIoRemain Set io status when enter standby mode.
 **        This parameter can be one of the following values:
 ** \arg   FALSE Set IO to Hi-z
 ** \arg   TRUE  Remain IO status
 ******************************************************************************
 */
void Lpm_GoToStandByMode(en_lpm_mode_t enMode, boolean_t bIoRemain)
{
    WRFSRDummyRead();
    WIFSRDummyRead();
    switch (enMode)
    {
        case StbSleepMode:
            CM0P_CORE_CSR &= 0xFFFFFFFFB;
            __WFI();
            break;
        case StbTimerMode:
            if( bIoRemain == 0 )
            {
                FM0P_CRG->STB_CTL = 0x1ACC0000;
            }
            else
            {
                FM0P_CRG->STB_CTL = 0x1ACC0010;
            }
            CM0P_CORE_CSR |= 0x00000004;
            __WFI();
            break;
        case StbStopMode:
            FM0P_DS->PMD_CTL &= ~0x01; /* RTCE=0 */
            if( bIoRemain == 0 )
            {
                FM0P_CRG->STB_CTL = 0x1ACC0002;
            }
            else
            {
                FM0P_CRG->STB_CTL = 0x1ACC0012;
            }
            CM0P_CORE_CSR |= 0x00000004;
            __WFI();
            break;    
        case StbRtcMode:
            FM0P_DS->PMD_CTL |= 0x01; /* RTCE=1 */
            if( bIoRemain == 0 )
            {
                FM0P_CRG->STB_CTL = 0x1ACC0002;
            }
            else
            {
                FM0P_CRG->STB_CTL = 0x1ACC0012;
            }
            CM0P_CORE_CSR |= 0x00000004;
            __WFI();
            break;
        case DeepStbRtcMode: 
            FM0P_DS->PMD_CTL |= 0x01; /* RTCE=1 */
            if( bIoRemain == 0 )
            {
                FM0P_CRG->STB_CTL = 0x1ACC0006;
            }
            else
            {
                FM0P_CRG->STB_CTL = 0x1ACC0016;
            }
            CM0P_CORE_CSR |= 0x00000004;
            __WFI();
            break;
        case DeepStbStopMode:
            FM0P_DS->PMD_CTL &= ~0x01; /* RTCE=0 */
            if( bIoRemain == 0 )
            {
                FM0P_CRG->STB_CTL = 0x1ACC0006;
            }
            else
            {
                FM0P_CRG->STB_CTL = 0x1ACC0016;
            }
            CM0P_CORE_CSR |= 0x00000004;
            __WFI();
            break;                       
        default:
            break;
    }
}

/*!
 ******************************************************************************
 ** \brief Configure return cause from deep standby mode
 **
 ** \param pstcCause Pointer to structure of return cause types
 **         
 ** \retval Ok  Deep standby mode is configured normally
 **
 ** \note Wakeup from WKUP0 pin is always enabled
 **
 ******************************************************************************
 */
en_result_t Lpm_ConfigDeepStbRetCause(stc_dstb_ret_cause_t* pstcCause)
{
    if(pstcCause->bRtcEn == 1)
    {
        bFM0P_DS_WIER_WRTCE = 1;
    }
    else
    {
        bFM0P_DS_WIER_WRTCE = 0;
    }
    
    if(pstcCause->bLvdEn == 1)
    {
        bFM0P_DS_WIER_WLVDE = 1;
    }
    else
    {
        bFM0P_DS_WIER_WLVDE = 0;
    }
    
    if(pstcCause->bWakeup1En == 1)
    {
        bFM0P_DS_WIER_WUE1 = 1;
    }
    else
    {
        bFM0P_DS_WIER_WUE1 = 0;
    }
    
    if(pstcCause->bWakeup2En == 1)
    {
        bFM0P_DS_WIER_WUE2 = 1;
    }
    else
    {
        bFM0P_DS_WIER_WUE2 = 0;
    }
    
    if(pstcCause->bWakeup3En == 1)
    {
        bFM0P_DS_WIER_WUE3 = 1;
    }
    else
    {
        bFM0P_DS_WIER_WUE3 = 0;
    }
    
    if(pstcCause->bWakeup4En == 1)
    {
        bFM0P_DS_WIER_WUE4 = 1;
    }
    else
    {
        bFM0P_DS_WIER_WUE4 = 0;
    }
    
    if(pstcCause->bWakeup5En == 1)
    {
        bFM0P_DS_WIER_WUE5 = 1;
    }
    else
    {
        bFM0P_DS_WIER_WUE5 = 0;
    }
    
    if(pstcCause->bCec0En == 1)
    {
        bFM0P_DS_WIER_WCEC0E = 1;
    }
    else
    {
        bFM0P_DS_WIER_WCEC0E = 0;
    }
    
    if(pstcCause->bCec1En == 1)
    {
        bFM0P_DS_WIER_WCEC1E = 1;
    }
    else
    {
        bFM0P_DS_WIER_WCEC1E = 0;
    }
    
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief Read deep standby mode return cause flag 
 **         
 ** \retval DeepStbNoFlag      No flag set
 ** \retval DeepStbInitx       Wakeup from deep standby mode by INITX reset
 ** \retval DeepStbLvdReset    Wakeup from deep standby mode by LVD reset
 ** \retval DeepStbRtcInt      Wakeup from deep standby mode by RTC interrupt
 ** \retval DeepStbLvdInt      Wakeup from deep standby mode by LVD interrupt
 ** \retval DeepStbWkupPin0    Wakeup from deep standby mode by WKUP0
 ** \retval DeepStbWkupPin1    Wakeup from deep standby mode by WKUP1
 ** \retval DeepStbWkupPin2    Wakeup from deep standby mode by WKUP2
 ** \retval DeepStbWkupPin3    Wakeup from deep standby mode by WKUP3
 ** \retval DeepStbWkupPin4    Wakeup from deep standby mode by WKUP4
 ** \retval DeepStbWkupPin5    Wakeup from deep standby mode by WKUP5
 ** \retval DeepStbCec0        Wakeup from deep standby mode by CEC0 interrupt
 ** \retval DeepStbCec1        Wakeup from deep standby mode by CEC1 interrupt
 ******************************************************************************
 */
en_dstb_ret_cause_t Lpm_ReadDeepStbRetCause(void)
{
    uint16_t rWIFSR,rWRFSR;
    rWRFSR = FM0P_DS->WRFSR;
    rWIFSR = FM0P_DS->WIFSR;
    
    if(rWRFSR & WRFSR_WINITX)
    {
        return DeepStbInitx;
    }
    
    if(rWRFSR & WIFSR_WLVDI)
    {
        return DeepStbLvdReset;
    }
    
    if(rWIFSR & WIFSR_WRTCI)
    {
        return DeepStbRtcInt;
    }
    else if(rWIFSR & WIFSR_WLVDI)
    {
        return DeepStbLvdInt;
    }
    else if(rWIFSR & WIFSR_WUI0)
    {
        return DeepStbWkupPin0;
    }
    else if(rWIFSR & WIFSR_WUI1)
    {
        return DeepStbWkupPin1;
    }
    else if(rWIFSR & WIFSR_WUI2)
    {
        return DeepStbWkupPin2;
    }
    else if(rWIFSR & WIFSR_WUI3)
    {
        return DeepStbWkupPin3;
    }
    else if(rWIFSR & WIFSR_WUI4)
    {
        return DeepStbWkupPin4;
    }
    else if(rWIFSR & WIFSR_WUI5)
    {
        return DeepStbWkupPin5;
    }
    else if(rWIFSR & WIFSR_WCEC0I)
    {
        return DeepStbCec0;
    }
    else if(rWIFSR & WIFSR_WCEC1I)
    {
        return DeepStbCec1;
    }
    return DeepStbNoFlag;
}

/*!
 ******************************************************************************
 ** \brief Set the valid level of wkup pin
 ** 
 ** \param enPinIndex Pin index
 ** \arg WkupPin1          WKUP1 pin index
 ** \arg WkupPin2          WKUP2 pin index
 ** \arg WkupPin3          WKUP3 pin index
 ** \arg WkupPin4          WKUP4 pin index
 ** \arg WkupPin5          WKUP5 pin index
 ** \param enLevel Wakeup pin valid value
 ** \arg WkupLowLevelValid     Set low as active level for WKUPx
 ** \arg WkupHighLevelValid    Set high as active level for WKUPx
 **         
 ** \retval Ok                     The valid level of wkup pin is set normally
 ** \retval ErrorInvalidParameter  Invalid value of enPinIndex
 **
 ******************************************************************************
 */
en_result_t Lpm_SetWkupPinLevel(en_dstb_wkup_pin_t enPinIndex, en_wkup_valid_level_t enLevel)
{
    if(enPinIndex > WkupPin5)
    {
        return ErrorInvalidParameter;
    }
  
    FM0P_DS->WILVR &= ~(1ul << enPinIndex);
    FM0P_DS->WILVR |= ((uint8_t)enLevel << enPinIndex);
    
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief Write the backup register
 ** 
 ** \param enRegIndex register index
 ** \arg BackupReg1     Back up register 1 index
 ** \arg BackupReg2     Back up register 2 index
 ** \arg BackupReg3     Back up register 3 index
 ** \arg BackupReg4     Back up register 4 index
 ** \arg BackupReg5     Back up register 5 index
 ** \arg BackupReg6     Back up register 6 index
 ** \arg BackupReg7     Back up register 7 index
 ** \arg BackupReg8     Back up register 8 index
 ** \arg BackupReg9     Back up register 9 index
 ** \arg BackupReg10    Back up register 10 index
 ** \arg BackupReg11    Back up register 11 index
 ** \arg BackupReg12    Back up register 12 index
 ** \arg BackupReg13    Back up register 13 index
 ** \arg BackupReg14    Back up register 14 index
 ** \arg BackupReg15    Back up register 15 index
 ** \arg BackupReg16    Back up register 16 index
 ** \param u8Data the data written into backup register
 **         
 ** \retval Ok The backup register written normally
 ** \retval ErrorInvalidParameter enRegIndex > BackupReg15
 **
 ******************************************************************************
 */
en_result_t Lpm_WriteBackupReg(en_dstb_bakup_reg_t enRegIndex, uint8_t u8Data)
{
    if(enRegIndex > BackupReg15)
    {
        return ErrorInvalidParameter;
    }
  
    *(uint8_t*)(LPCM_BACKUP_REG_BASE + (uint8_t)enRegIndex) = u8Data;
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief Read the backup register
 ** 
 ** \param enRegIndex Backup register index
 ** \arg BackupReg1     Back up register 1 index
 ** \arg BackupReg2     Back up register 2 index
 ** \arg BackupReg3     Back up register 3 index
 ** \arg BackupReg4     Back up register 4 index
 ** \arg BackupReg5     Back up register 5 index
 ** \arg BackupReg6     Back up register 6 index
 ** \arg BackupReg7     Back up register 7 index
 ** \arg BackupReg8     Back up register 8 index
 ** \arg BackupReg9     Back up register 9 index
 ** \arg BackupReg10    Back up register 10 index
 ** \arg BackupReg11    Back up register 11 index
 ** \arg BackupReg12    Back up register 12 index
 ** \arg BackupReg13    Back up register 13 index
 ** \arg BackupReg14    Back up register 14 index
 ** \arg BackupReg15    Back up register 15 index
 ** \arg BackupReg16    Back up register 16 index
 **         
 ** \return Backup register data
 **
 ******************************************************************************
 */
uint8_t Lpm_ReadBackupReg(en_dstb_bakup_reg_t enRegIndex)
{
    return *(uint8_t*)(LPCM_BACKUP_REG_BASE + (uint8_t)enRegIndex);
}

/*!
 ******************************************************************************
 ** \brief Configue the sub clock supply 
 ** 
 ** \param bSupplyCec Whether to supply sub clock to CEC
 ** \param bSupplyRtc Whether to supply sub clock to RTC
 **         
 ** \retval Ok The sub clock supply is configured normally
 **
 ** \note Not all FM0+ product has this function, please check it in the 
 **       product manual.
 **
 ******************************************************************************
 */
en_result_t Lpm_ConfigSubClk(boolean_t bSupplyCec, boolean_t bSupplyRtc)
{
    if(bSupplyCec == TRUE)
    {
        bFM0P_DS_RCK_CTL_CECCKE = 1;
    }
    else
    {
        bFM0P_DS_RCK_CTL_CECCKE = 0;
    }
    
    if(bSupplyRtc == TRUE)
    {
        bFM0P_DS_RCK_CTL_RTCCKE = 1;
    }
    else
    {
        bFM0P_DS_RCK_CTL_RTCCKE = 0;
    }
    
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief Configure deep standby mode RAM retention
 ** 
 ** \param bRamRetain Whether to retain data in the RAM
 **         
 ** \retval Ok Deep standby mode RAM retention set
 **
 ** \note Not all FM0+ product has this function, please check it in the 
 **       product manual.
 **
 ******************************************************************************
 */
en_result_t Lpm_ConfigDeepStbRAMRetention(boolean_t bRamRetain)
{
    if(bRamRetain == FALSE)
    {
        FM0P_DS->DSRAMR &= ~0x03u; 
    }
    else
    {
        FM0P_DS->DSRAMR |= 0x03u; 
    }
    return Ok;
}

//@}

#endif // #if (defined(PDL_PERIPHERAL_ENABLE_LPM))

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
