/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file i2c_at24cxx.h
 **
 ** AT24CXX driver file header file
 **
 ** The AT24C02 is tested in the example, make following pin connection first.
 **
 ** --------------------------------------------
 ** | MCU             AT24C02                   |
 ** |-------------------------------------------
 ** | Pin Name        Pin Count     Pin Name    |   
 ** | GND             1             A0          |
 ** | GND             2             A1          |   
 ** | GND             3             A2          |
 ** | GND             4             GND         |
 ** | SOT1_1          5             SDA         |
 ** | SCK1_1          6             SCL         |
 ** | -               7             WP          |
 ** | VCC             8             VCC         |
 ** --------------------------------------------
 **
 ** History:
 **   - 2014-03-10  1.0  Edison Zhang        First release version.
 **
 ******************************************************************************/
#ifndef _I2C_AT24CXX_H_
#define _I2C_PAT24CXX_H_

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_user.h"
#include "at24cxx_config.h"
#include "mfs\mfs.h"   
#include "gpio\gpio.h" 

#if (PDL_UTILITY_ENABLE_I2C_POLLING_AT24CXX == PDL_ON) || \
    (PDL_UTILITY_ENABLE_I2C_INT_AT24CXX == PDL_ON)

/**
 ******************************************************************************
 ** \defgroup I2cAt24cxxGroup I2C AT24CXX Function
 **
 ** Provided functions of AT24CXX driver:
 **
 ** - At24cxx_Init()
 ** - At24cxx_ByteWrite()
 ** - At24cxx_PageWrite()
 ** - At24cxx_CurrentAddrRead()
 ** - At24cxx_RandomRead()
 ** - At24cxx_SequentialRead()
 ** - At24cxx_Delayms()     
 **
 ** At24cxx_Init() initializes the I2C channel to access AT24CXX.
 **
 ** At24cxx_ByteWrite() writes a byte at a ramdon address in the EEPROM memory.
 **
 ** At24cxx_PageWrite() can write up to 1 page bytes. 
 **
 ** At24cxx_CurrentAddrRead() reads the data at the current address.
 **
 ** At24cxx_RandomRead() can read a byte at a ramdon address in the EEPROM 
 ** memory.
 **
 ** At24cxx_SequentialRead() can read as many bytes as whole EEPROM memory,
 ** which should follow At24cxx_CurrentAddrRead() or At24cxx_RandomRead().      
 **
 ******************************************************************************/ 

//@{      
      
/******************************************************************************
 * Global definitions
 ******************************************************************************/
/** AT24CXX device address [A0=A1=A2=0] */
#define AT24CXX_7BIT_DEVICE_ADDR        (0x50u)                     
   
/** AT24CXX page size    */
#if (PDL_EEPROM_DEVICE == PDL_AT24C01) || \
    (PDL_EEPROM_DEVICE == PDL_AT24C02)
#define AT24CXX_PAGE_SIZE                      8
#elif (PDL_EEPROM_DEVICE == PDL_AT24C04) || \
      (PDL_EEPROM_DEVICE == PDL_AT24C08) || \
      (PDL_EEPROM_DEVICE == PDL_AT24C16)
#define AT24CXX_PAGE_SIZE                      16
#elif (PDL_EEPROM_DEVICE == PDL_AT24C32) || \
      (PDL_EEPROM_DEVICE == PDL_AT24C64)
#define AT24CXX_PAGE_SIZE                      32
#endif

/** AT24CXX address byte length  */        
#if (PDL_EEPROM_DEVICE == PDL_AT24C01) || \
    (PDL_EEPROM_DEVICE == PDL_AT24C02) || \
    (PDL_EEPROM_DEVICE == PDL_AT24C04) || \
    (PDL_EEPROM_DEVICE == PDL_AT24C08) || \
    (PDL_EEPROM_DEVICE == PDL_AT24C16) 
#define AT24CXX_ADDR_LENGTH                    1
#else
#define AT24CXX_ADDR_LENGTH                    2
#endif      
        
/** I2C IO function initialization */       
#define InitI2cIo(void)  {SetPinFunc_SOT1_1();SetPinFunc_SCK1_1();}

/** I2C channel used for AT24C02   */
#define EE_I2C_CH        &I2C1  
      
/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif
en_result_t At24cxx_Init(void); 
en_result_t At24cxx_ByteWrite(uint8_t u8DevAddr, uint16_t u16Addr, uint8_t u8Data);
en_result_t At24cxx_PageWrite(uint8_t u8DevAddr, uint16_t u16PageAddr, uint8_t* pu8Data, uint8_t u8Size);
en_result_t At24cxx_CurrentAddrRead(uint8_t u8DevAddr, uint8_t* pu8CurData);
en_result_t At24cxx_RandomRead(uint8_t u8DevAddr, uint16_t u16Addr, uint8_t* pu8Data);
en_result_t At24cxx_SequentialRead(uint8_t u8DevAddr, uint8_t* pu8Data, uint8_t u8Size);
void At24cxx_Delayms(uint32_t u32Cnt);
#ifdef __cplusplus
}
#endif

//@} 

#endif // #if (PDL_UTILITY_ENABLE_I2C_POLLING_AT24CXX == PDL_ON)

#endif //  _I2C_AT24CXX_H_

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
