/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/*****************************************************************************/
/** \file srecordparser.c
 **
 ** API to parse a motorola S-Record file, like *.mhx or *.srec
 **
 ** History:
 **   - 2012-03-16    1.0  MSc  First version
 **   - 2014-06-25    1.1  MSc  Created univeral SRecord parser
 *****************************************************************************/

#include "srecordparser.h"

dataread_func_ptr_t pfnDataRead = NULL;

/**
 ******************************************************************************
 ** \brief  Set data receive handle, can be file, UART, USB
 **
 ** \param  pfnHandle  Data Read Callback
 **
 ******************************************************************************/
void SRecordParser_SetReadDataHandle(dataread_func_ptr_t pfnHandle)
{
    pfnDataRead = pfnHandle;
}

/**
 ******************************************************************************
 ** \brief  Convert string ex format to uint8_t (byte)
 **
 ** \param  pointer to string
 ** \return byte value
 ******************************************************************************/
uint8_t SRecordParser_HexToByte(char_t* pcData)
{
    uint8_t u8Temp = 0;
    if ((pcData[0] >= '0') & (pcData[0] <= '9'))
    {
        u8Temp += ((uint8_t)(pcData[0] - '0')) << 4;
    }
    if ((pcData[0] >= 'A') & (pcData[0] <= 'F'))
    {
        u8Temp += ((uint8_t)(pcData[0] - 'A' + 10)) << 4;
    }
    
    if ((pcData[1] >= '0') & (pcData[1] <= '9'))
    {
        u8Temp += ((uint8_t)(pcData[1] - '0'));
    }
    if ((pcData[1] >= 'A') & (pcData[1] <= 'F'))
    {
        u8Temp += ((uint8_t)(pcData[1] - 'A' + 10));
    }
    return u8Temp;
}


/**
 ******************************************************************************
 ** \brief  Read line of S-Record file
 **
 ** \param  pu8Data        buffer to write in converted data
 ** \param  pu8DataLength  pointer in which the datalength will be written
 ** \return start address, in case of error / end of file, *pu8DataLength = 0,
 **         return value contains the errorcode:
 **         SRECORDPARSER_CRCERROR in case of CRC error
 **         SRECORDPARSER_ENDOFFILE in case of end of file
 ******************************************************************************/
uint32_t SRecordParser_ReadLine(uint8_t* pu8Data, uint8_t* pu8DataLength)
{
    char_t pcTemp[2];
    uint32_t uintBytesRead;
    uint8_t u8State = 0;
    uint8_t u8SRecordType = 0;
    uint8_t u8ByteCount = 0;
    uint32_t u32StartAddress = 0;
    uint8_t u8CRC = 0;
    *pu8DataLength = 0;
    if (pfnDataRead == NULL) return SRECORDPARSER_ENDOFFILE;
    while(1)
    {
        pcTemp[0] = 0;
        pcTemp[1] = 0;
        
        pfnDataRead((uint8_t*)pcTemp,2,&uintBytesRead);
        /*puts("Data ");
        putch(pcTemp[0]);
        putch(pcTemp[1]);
        puts("\n");*/
        if (uintBytesRead == 0)
        {
            if (*pu8DataLength == 0)
            {
                u32StartAddress = SRECORDPARSER_ENDOFFILE;
            }
            break;
        }
        if ((pcTemp[0] == '\r') && (pcTemp[1] == '\n'))
        {
            break;
        }
        if (pcTemp[0] == 'S')
        {
            u8State = 0;
        }
        if ((u8State > 0) && (u8State < 7))
        {
          u8CRC += SRecordParser_HexToByte(pcTemp);
        }
        switch(u8State)
        {
            case 0: //state 0, wait for S-Record type
                if (pcTemp[0] == 'S')
                {
                   u8SRecordType = (uint8_t)(pcTemp[1] - '0');
                   u8State++;
                }
                if ((u8SRecordType < 1) || (u8SRecordType > 3))
                {
                  u8SRecordType = 0;
                  u8State = 0;
                } else
                {
                  /*puts("New S-Record: ");
                  putch(u8SRecordType + '0');
                  puts("\n");*/
                }
                break;
            case 1: //state 1, read length of data
              u8ByteCount = SRecordParser_HexToByte(pcTemp);
              u8ByteCount -= (u8SRecordType + 2); // remove checksum byte and address bytes
              *pu8DataLength = u8ByteCount;
              u8State++;
              break;
            case 2: //state 2..5 read address bytes
              u32StartAddress = 0;
            case 3:
            case 4:
            case 5:
              
              u32StartAddress += SRecordParser_HexToByte(pcTemp)  << (8 * (u8SRecordType - (u8State - 2)));
              if ((u8SRecordType - (u8State - 2)) == 0)
              {
                  u8State = 5;
              }
              u8State++;
              break;
            case 6: //state 6, read data bytes
              u8ByteCount--;
              if (u8ByteCount == 0)
              {
                  u8State++;
              }
              *pu8Data = SRecordParser_HexToByte(pcTemp);
              pu8Data++;
              break;
            case 7: //state 7, read CRC
              if ((0xFF ^ SRecordParser_HexToByte(pcTemp)) != u8CRC)
              {
                //puts("CRC Error");
                u32StartAddress = SRECORDPARSER_CRCERROR;
                *pu8DataLength = 0;
                //f_sync(&filHandle);
                return u32StartAddress;
              }
              else
              {
                //puts("CRC OK");
              }
              break;
            default:
              break;
        }
    }
    /*puts("Address: 0x");
    puthex(u32StartAddress,8);
    puts("\n");*/
    return u32StartAddress;
    
}

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/


