/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file ringbuffer.c
 **
 ** Ring Buffer Implementation
 **
 ** History:
 **   - 2014-06-26  1.0  MSc        First release version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/

#include "ringbuffer.h"

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/

/**
 ******************************************************************************
 ** \brief  Initialize Ring Buffer
 **
 ** \param  pstcHandle  Ring Buffer Handle
 ** \return Ok or ErrorUninitialized if pstcHandle is NULL
 ******************************************************************************/
en_result_t RingBuffer_Init(stc_ringbuffer_t* pstcHandle)
{
    if (pstcHandle == NULL) return ErrorUninitialized;
    pstcHandle->u32DataWritePos = 0;
    pstcHandle->u32DataReadPos = 0;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Add new data to ring buffer
 **
 ** \param  pstcHandle    Ring Buffer Handle
 **
 ** \param  pu8Buffer     Buffer to add
 **
 ** \param u32DataLength  Size of buffer to add
 **
 ** \param pfnOverwriteWaitCallback Wait routine, if returns TRUE, overwrite will executed
 **                                 If this is NULL, overwrite will always work
 **
 ** \return Ok or ErrorUninitialized if pstcHandle is NULL
 ******************************************************************************/
en_result_t RingBuffer_Add(stc_ringbuffer_t* pstcHandle, uint8_t* pu8Buffer, uint32_t u32DataLength, ringbuffer_datawait_func_ptr_t pfnOverwriteWaitCallback)
{
    uint32_t i;
    if (pstcHandle == NULL) return ErrorUninitialized;
    for(i = 0; i < u32DataLength;i++)
    {
        if (pstcHandle->u32DataWritePos >= pstcHandle->u32Size)
        {
            pstcHandle->u32DataWritePos = 0;

            if (pfnOverwriteWaitCallback != NULL)
            {
                while(pstcHandle->u32DataWritePos == pstcHandle->u32DataReadPos)
                {
                    if (pfnOverwriteWaitCallback() == TRUE) 
                    {
                        break;
                    }
                }
            }

        }
        pstcHandle->pu8Buffer[pstcHandle->u32DataWritePos] = pu8Buffer[i];
        pstcHandle->u32DataWritePos++;
        if (pfnOverwriteWaitCallback != NULL)
        {
            while(pstcHandle->u32DataWritePos == pstcHandle->u32DataReadPos)
            {
                if (pfnOverwriteWaitCallback() == TRUE) 
                {
                    break;
                }
            }
        }

        
    }
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Read data from ring buffer
 **
 ** \param  pstcHandle    Ring Buffer Handle
 **
 ** \param  pu8Buffer     Buffer to read in
 **
 ** \param u32DataLength  Size of data to be read
 **
 ** \param pu32DataRead   Returns read data size
 **
 ** \param pfnDataWaitCallback      Wait routine if data is not ready, if returns TRUE, read will stopped
 **                                 If this is NULL only maximum available data will be read
 **
 ** \return Ok or ErrorUninitialized if pstcHandle is NULL
 ******************************************************************************/
en_result_t RingBuffer_Read(stc_ringbuffer_t* pstcHandle, uint8_t* pu8Buffer, uint32_t u32DataLength,uint32_t* pu32DataRead, ringbuffer_datawait_func_ptr_t pfnDataWaitCallback)
{
    uint32_t i;
    volatile uint32_t u32Timeout;
    *pu32DataRead = 0;
    if (pstcHandle == NULL) return ErrorUninitialized;
    for(i = 0;i < u32DataLength;i++)
    {
        if (pstcHandle->u32DataReadPos >= pstcHandle->u32Size)
        {
            pstcHandle->u32DataReadPos = 0;
        }
        if (pfnDataWaitCallback != NULL)
        {
            while(pstcHandle->u32DataWritePos == pstcHandle->u32DataReadPos)
            {
                if (pfnDataWaitCallback() == TRUE) 
                {
                    return Ok;
                }
            }
        } else if (pstcHandle->u32DataWritePos == pstcHandle->u32DataReadPos)
        {
            return Ok;
        }
        *pu32DataRead = *pu32DataRead + 1;
        pu8Buffer[i] = pstcHandle->pu8Buffer[pstcHandle->u32DataReadPos];
        pstcHandle->u32DataReadPos++;
    }
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Read data from ring buffer without changing read pointer
 **
 ** \param  pstcHandle    Ring Buffer Handle
 **
 ** \param  pu8Buffer     Buffer to read in
 **
 ** \param u32DataLength  Size of data to be read
 **
 ** \param pu32DataRead   Returns read data size
 **
 ** \param pfnDataWaitCallback      Wait routine if data is not ready, if returns TRUE, read will stopped
 **                                 If this is NULL only maximum available data will be read
 **
 ** \return Ok or ErrorUninitialized if pstcHandle is NULL
 ******************************************************************************/
en_result_t RingBuffer_SniffRead(stc_ringbuffer_t* pstcHandle, uint8_t* pu8Buffer, uint32_t u32DataLength,uint32_t* pu32DataRead, ringbuffer_datawait_func_ptr_t pfnDataWaitCallback)
{
    uint32_t i;
    volatile uint32_t u32Timeout;
    *pu32DataRead = 0;
    if (pstcHandle == NULL) return ErrorUninitialized;
    for(i = 0;i < u32DataLength;i++)
    {
        if (pstcHandle->u32DataReadPos >= pstcHandle->u32Size)
        {
            pstcHandle->u32DataReadPos = 0;
        }
        if (pfnDataWaitCallback != NULL)
        {
            while(pstcHandle->u32DataWritePos == pstcHandle->u32DataReadPos)
            {
                if (pfnDataWaitCallback() == TRUE) 
                {
                    return Ok;
                }
            }
        } else if (pstcHandle->u32DataWritePos == pstcHandle->u32DataReadPos)
        {
            return Ok;
        }
        *pu32DataRead = *pu32DataRead + 1;
        pu8Buffer[i] = pstcHandle->pu8Buffer[pstcHandle->u32DataReadPos + i];
    }
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Set Read Position to Write Position
 **
 ** \param  pstcHandle    Ring Buffer Handle
 **
 **
 ** \return Ok or ErrorUninitialized if pstcHandle is NULL
 ******************************************************************************/

en_result_t RingBuffer_SetReadToWritePosition(stc_ringbuffer_t* pstcHandle)
{
    if (pstcHandle == NULL) return ErrorUninitialized;
    pstcHandle->u32DataReadPos = pstcHandle->u32DataWritePos;
    return Ok;
}

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/

